#!/usr/bin/python
#
# Copyright (c) 2023 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_vmssnetworkinterface_info

version_added: "1.15.0"

short_description: Get information about network interface in virtul machine scale

description:
    - Get information about network interface in virtual machine scale set.

options:
    name:
        description:
            - The name of the network interface.
            - If configure I(name), you must set the parameters I(vm_index).
        type: str
    vmss_name:
        description:
            - The name of the virtual machine scale set.
        type: str
        required: True
    vm_index:
        description:
            - The virtual machine index, such as I(vm_index=0).
        type: str
    resource_group:
        description:
            - Name of the resource group.
        type: str
        required: True

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)

'''

EXAMPLES = '''
- name: Get information by the network name
  azure_rm_vmssnetworkinterface_info:
    resource_group: myResourceGroup
    name: nic001
    vmss_name: testVMSS
    vm_index: 0

- name: Get all network interface information in virtual machine scale set
  azure_rm_vmssnetworkinterface_info:
    resource_group: myResourceGroup
    vmss_name: testVMSS

- name: Get all network interface information in the same virtual machine index.
  azure_rm_vmssnetworkinterface_info:
    resource_group: myResourceGroup
    vmss_name: testVMSS
    vm_index: 1
'''

RETURN = '''
vmss_networkinterfaces:
    description:
        - List of network interface dicts. Each dict contains parameters can be passed to M(azure.azcollection.azure_rm_vmssnetworkinterface) module.
    type: complex
    returned: always
    contains:
        id:
            description:
                - Id of the network interface.
            returned: always
            type: str
            sample: "/subscriptions/xxx-xxx/resourceGroups/RG/providers/Microsoft.Compute/virtualMachineScaleSets/fredvmss/virtualMachines/1/networkInterfaces/nic01"
        resource_group:
            description:
                - Name of a resource group where the network interface exists.
            returned: always
            type: str
            sample: RG
        name:
            description:
                - Name of the network interface.
            type: str
            returned: always
            sample: nic01
        location:
            description:
                - Azure location.
            type: str
            returned: always
            sample: eastus
        virtual_network:
            description:
                - An existing virtual network with which the network interface will be associated.
                - It is a dict which contains I(name) and I(resource_group) of the virtual network.
            type: dict
            returned: always
            sample: {"name": "vnet01", "resource_group": "RG"}
        subnet:
            description:
                - Name of an existing subnet within the specified virtual network.
            type: str
            returned: always
            sample: default
        tags:
            description:
                - Tags of the network interface.
            type: dict
            returned: always
            sample: {"key1": "value1"}
        ip_configurations:
            description:
                - List of IP configurations, if contains multiple configurations.
            type: complex
            returned: always
            contains:
                name:
                    description:
                        - Name of the IP configuration.
                    type: str
                    returned: always
                    sample: defaultIpConfiguration
                private_ip_address:
                    description:
                        - Private IP address for the IP configuration.
                    type: str
                    returned: always
                    sample: 10.3.0.5
                private_ip_allocation_method:
                    description:
                        - Private IP allocation method.
                    returned: always
                    type: str
                    sample: Dynamic
                public_ip_address:
                    description:
                        - Name of the public IP address. None for disable IP address.
                    returned: always
                    type: str
                    sample: null
                public_ip_allocation_method:
                    description:
                        - Public IP allocation method.
                    returned: always
                    type: str
                    sample: null
                load_balancer_backend_address_pools:
                    description:
                        - List of existing load-balancer backend address pools associated with the network interface.
                    returned: always
                    type: str
                    sample: null
                application_gateway_backend_address_pools:
                    description:
                        - List of existing application gateway backend address pools associated with the network interface.
                    returned: always
                    type: str
                    sample: null
                primary:
                    description:
                        - Whether the IP configuration is the primary one in the list.
                    returned: always
                    type: bool
                    sample: True
                application_security_groups:
                    description:
                        - List of Application security groups.
                    returned: always
                    type: str
                    sample: /subscriptions/<subsid>/resourceGroups/<rg>/providers/Microsoft.Network/applicationSecurityGroups/myASG
        enable_accelerated_networking:
            description:
                - Specifies whether the network interface should be created with the accelerated networking feature or not.
            type: bool
            returned: always
            sample: True
        create_with_security_group:
            description:
                - Specifies whether a default security group should be be created with the NIC. Only applies when creating a new NIC.
            type: bool
            returned: always
            sample: True
        security_group:
            description:
                - A security group resource ID with which to associate the network interface.
            type: str
            returned: always
            sample: /subscriptions/xxx-xxx/resourceGroups/RG/providers/Microsoft.Network/networkSecurityGroups/nic01
        enable_ip_forwarding:
            description:
                - Whether to enable IP forwarding
            type: bool
            returned: always
            sample: True
        dns_servers:
            description:
                - Which DNS servers should the NIC lookup.
                - List of IP addresses.
            type: list
            returned: always
            sample: []
        mac_address:
            description:
                - The MAC address of the network interface.
            type: str
            returned: always
            sample: 00-0D-3A-17-EC-36
        provisioning_state:
            description:
                - The provisioning state of the network interface.
            type: str
            returned: always
            sample: Succeeded
        dns_settings:
            description:
                - The DNS settings in network interface.
            type: complex
            returned: always
            contains:
                dns_servers:
                    description:
                        - List of DNS servers IP addresses.
                    returned: always
                    type: list
                    sample: []
                applied_dns_servers:
                    description:
                        - If the VM that uses this NIC is part of an Availability Set, then this list will have the union of all DNS servers
                          from all NICs that are part of the Availability Set. This property is what is configured on each of those VMs.
                    returned: always
                    type: list
                    sample: []
                internal_dns_name_label:
                    description:
                        - Relative DNS name for this NIC used for internal communications between VMs in the same virtual network.
                    returned: always
                    type: str
                    sample: null
                internal_fqdn:
                    description:
                        - Fully qualified DNS name supporting internal communications between VMs in the same virtual network.
                    returned: always
                    type: str
                    sample: null
'''  # NOQA
try:
    from azure.core.exceptions import ResourceNotFoundError
except Exception:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase, azure_id_to_dict


AZURE_OBJECT_CLASS = 'VMSSNetworkInterface'


def nic_to_dict(nic):
    ip_configurations = [
        dict(
            name=config.name,
            private_ip_address=config.private_ip_address,
            private_ip_allocation_method=config.private_ip_allocation_method,
            primary=config.primary if config.primary else False,
            load_balancer_backend_address_pools=([item.id for item in config.load_balancer_backend_address_pools]
                                                 if config.load_balancer_backend_address_pools else None),
            application_gateway_backend_address_pools=([item.id for item in config.application_gateway_backend_address_pools]
                                                       if config.application_gateway_backend_address_pools else None),
            public_ip_address=config.public_ip_address.id if config.public_ip_address else None,
            public_ip_allocation_method=config.public_ip_address.public_ip_allocation_method if config.public_ip_address else None,
            application_security_groups=([asg.id for asg in config.application_security_groups]
                                         if config.application_security_groups else None)
        ) for config in nic.ip_configurations
    ]
    config = nic.ip_configurations[0] if len(nic.ip_configurations) > 0 else None
    subnet_dict = azure_id_to_dict(config.subnet.id) if config and config.subnet else None
    subnet = subnet_dict.get('subnets') if subnet_dict else None
    virtual_network = dict(
        resource_group=subnet_dict.get('resourceGroups'),
        name=subnet_dict.get('virtualNetworks')) if subnet_dict else None
    return dict(
        id=nic.id,
        resource_group=azure_id_to_dict(nic.id).get('resourceGroups'),
        name=nic.name,
        subnet=subnet,
        virtual_network=virtual_network,
        location=nic.location,
        tags=nic.tags,
        security_group=nic.network_security_group.id if nic.network_security_group else None,
        dns_settings=dict(
            dns_servers=nic.dns_settings.dns_servers,
            applied_dns_servers=nic.dns_settings.applied_dns_servers,
            internal_dns_name_label=nic.dns_settings.internal_dns_name_label,
            internal_fqdn=nic.dns_settings.internal_fqdn
        ),
        ip_configurations=ip_configurations,
        mac_address=nic.mac_address,
        enable_ip_forwarding=nic.enable_ip_forwarding,
        provisioning_state=nic.provisioning_state,
        enable_accelerated_networking=nic.enable_accelerated_networking,
        dns_servers=nic.dns_settings.dns_servers,
    )


class AzureRMVMSSNetworkInterfaceInfo(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type='str'),
            resource_group=dict(type='str', required=True),
            vmss_name=dict(type='str', required=True),
            vm_index=dict(type='str'),
        )

        self.results = dict(
            changed=False,
        )

        self.name = None
        self.resource_group = None
        self.vmss_name = None
        self.vm_index = None

        super(AzureRMVMSSNetworkInterfaceInfo, self).__init__(self.module_arg_spec,
                                                              supports_check_mode=True,
                                                              supports_tags=False,
                                                              facts_module=True
                                                              )

    def exec_module(self, **kwargs):
        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        results = []
        if self.name is not None:
            if self.vm_index is not None:
                results = self.get_item()
            else:
                self.fail("Parameter error: vm_index required when filtering by name.")
        elif self.vm_index is not None:
            results = self.list_vm_index()
        else:
            results = self.list_vmss()

        self.results['vmss_networkinterfaces'] = self.to_dict_list(results)
        return self.results

    def get_item(self):
        res = None
        self.log("Get the specified network interface in a virtual machine scale set.")
        try:
            res = self.network_client.network_interfaces.get_virtual_machine_scale_set_network_interface(resource_group_name=self.resource_group,
                                                                                                         virtual_machine_scale_set_name=self.vmss_name,
                                                                                                         virtualmachine_index=self.vm_index,
                                                                                                         network_interface_name=self.name)
        except ResourceNotFoundError:
            pass

        return [res] if res is not None else []

    def list_vm_index(self):
        try:
            res = self.network_client.network_interfaces.list_virtual_machine_scale_set_vm_network_interfaces(resource_group_name=self.resource_group,
                                                                                                              virtual_machine_scale_set_name=self.vmss_name,
                                                                                                              virtualmachine_index=self.vm_index)
            return list(res)
        except Exception as exc:
            self.fail("Error listing by resource group {0} - {1}".format(self.resource_group, str(exc)))

    def list_vmss(self):
        self.log('List all')
        try:
            response = self.network_client.network_interfaces.list_virtual_machine_scale_set_network_interfaces(resource_group_name=self.resource_group,
                                                                                                                virtual_machine_scale_set_name=self.vmss_name)
            return list(response)
        except Exception as exc:
            self.fail("Error listing all - {0}".format(str(exc)))

    def to_dict_list(self, raws):
        return [nic_to_dict(item) for item in raws] if raws else []


def main():
    AzureRMVMSSNetworkInterfaceInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: sda_device_info
short_description: Information module for Sda Device Info
description:
- This module represents an alias of the module sda_device_v1_info
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  deviceManagementIpAddress:
    description:
    - DeviceManagementIpAddress query parameter.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for SDA GetDeviceInfoFromSDAFabricV1
  description: Complete reference of the GetDeviceInfoFromSDAFabricV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-device-info-from-sda-fabric
notes:
  - SDK Method used are
    sda.Sda.get_device_info,

  - Paths used are
    get /dna/intent/api/v1/business/sda/device,
  - It should be noted that this module is an alias of sda_device_v1_info

"""

EXAMPLES = r"""
- name: Get all Sda Device Info
  cisco.dnac.sda_device_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    deviceManagementIpAddress: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "status": "string",
      "description": "string",
      "name": "string",
      "roles": [
        "string"
      ],
      "deviceManagementIpAddress": "string",
      "siteHierarchy": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: pnp_device_info
short_description: Information module for Pnp Device Info
description:
- This module represents an alias of the module pnp_device_v1_info
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  limit:
    description:
    - Limit query parameter. Limits number of results.
    type: int
  offset:
    description:
    - Offset query parameter. Index of first result.
    type: int
  sort:
    description:
    - Sort query parameter. Comma seperated list of fields to sort on.
    elements: str
    type: list
  sortOrder:
    description:
    - SortOrder query parameter. Sort Order Ascending (asc) or Descending (des).
    type: str
  serialNumber:
    description:
    - SerialNumber query parameter. Device Serial Number.
    elements: str
    type: list
  state_:
    description:
    - State query parameter. Device State.
    elements: str
    type: list
  onbState:
    description:
    - OnbState query parameter. Device Onboarding State.
    elements: str
    type: list
  name:
    description:
    - Name query parameter. Device Name.
    elements: str
    type: list
  pid:
    description:
    - Pid query parameter. Device ProductId.
    elements: str
    type: list
  source:
    description:
    - Source query parameter. Device Source.
    elements: str
    type: list
  workflowId:
    description:
    - WorkflowId query parameter. Device Workflow Id.
    elements: str
    type: list
  workflowName:
    description:
    - WorkflowName query parameter. Device Workflow Name.
    elements: str
    type: list
  smartAccountId:
    description:
    - SmartAccountId query parameter. Device Smart Account.
    elements: str
    type: list
  virtualAccountId:
    description:
    - VirtualAccountId query parameter. Device Virtual Account.
    elements: str
    type: list
  lastContact:
    description:
    - LastContact query parameter. Device Has Contacted lastContact > 0.
    type: bool
  macAddress:
    description:
    - MacAddress query parameter. Device Mac Address.
    type: str
  hostname:
    description:
    - Hostname query parameter. Device Hostname.
    type: str
  siteName:
    description:
    - SiteName query parameter. Device Site Name.
    type: str
  id:
    description:
    - Id path parameter.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Device Onboarding (PnP) GetDeviceByIdV1
  description: Complete reference of the GetDeviceByIdV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-device-by-id
- name: Cisco DNA Center documentation for Device Onboarding (PnP) GetDeviceListSiteManagementV1
  description: Complete reference of the GetDeviceListSiteManagementV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-device-list-site-management
notes:
  - SDK Method used are
    device_onboarding_pnp.DeviceOnboardingPnp.get_device_by_id_v1,
    device_onboarding_pnp.DeviceOnboardingPnp.get_device_list,

  - Paths used are
    get /dna/intent/api/v1/onboarding/pnp-device,
    get /dna/intent/api/v1/onboarding/pnp-device/{id},
  - It should be noted that this module is an alias of pnp_device_v1_info

"""

EXAMPLES = r"""
- name: Get all Pnp Device Info
  cisco.dnac.pnp_device_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    limit: 0
    offset: 0
    sort: []
    sortOrder: string
    serialNumber: []
    state_: []
    onbState: []
    name: []
    pid: []
    source: []
    workflowId: []
    workflowName: []
    smartAccountId: []
    virtualAccountId: []
    lastContact: True
    macAddress: string
    hostname: string
    siteName: string
  register: result

- name: Get Pnp Device Info by id
  cisco.dnac.pnp_device_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    id: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "_id": "string",
      "deviceInfo": {
        "source": "string",
        "serialNumber": "string",
        "stack": true,
        "mode": "string",
        "state": "string",
        "location": {
          "siteId": "string",
          "address": "string",
          "latitude": "string",
          "longitude": "string",
          "altitude": "string"
        },
        "description": "string",
        "onbState": "string",
        "authenticatedMicNumber": "string",
        "authenticatedSudiSerialNo": "string",
        "capabilitiesSupported": [
          "string"
        ],
        "featuresSupported": [
          "string"
        ],
        "cmState": "string",
        "firstContact": 0,
        "lastContact": 0,
        "macAddress": "string",
        "pid": "string",
        "deviceSudiSerialNos": [
          "string"
        ],
        "lastUpdateOn": 0,
        "workflowId": "string",
        "workflowName": "string",
        "projectId": "string",
        "projectName": "string",
        "deviceType": "string",
        "agentType": "string",
        "imageVersion": "string",
        "fileSystemList": [
          {
            "type": "string",
            "writeable": true,
            "freespace": 0,
            "name": "string",
            "readable": true,
            "size": 0
          }
        ],
        "pnpProfileList": [
          {
            "profileName": "string",
            "discoveryCreated": true,
            "createdBy": "string",
            "primaryEndpoint": {
              "port": 0,
              "protocol": "string",
              "ipv4Address": {},
              "ipv6Address": {},
              "fqdn": "string",
              "certificate": "string"
            },
            "secondaryEndpoint": {
              "port": 0,
              "protocol": "string",
              "ipv4Address": {},
              "ipv6Address": {},
              "fqdn": "string",
              "certificate": "string"
            }
          }
        ],
        "imageFile": "string",
        "httpHeaders": [
          {
            "key": "string",
            "value": "string"
          }
        ],
        "neighborLinks": [
          {
            "localInterfaceName": "string",
            "localShortInterfaceName": "string",
            "localMacAddress": "string",
            "remoteInterfaceName": "string",
            "remoteShortInterfaceName": "string",
            "remoteMacAddress": "string",
            "remoteDeviceName": "string",
            "remotePlatform": "string",
            "remoteVersion": "string"
          }
        ],
        "lastSyncTime": 0,
        "ipInterfaces": [
          {
            "status": "string",
            "macAddress": "string",
            "ipv4Address": {},
            "ipv6AddressList": [
              {}
            ],
            "name": "string"
          }
        ],
        "hostname": "string",
        "authStatus": "string",
        "stackInfo": {
          "supportsStackWorkflows": true,
          "isFullRing": true,
          "stackMemberList": [
            {
              "serialNumber": "string",
              "state": "string",
              "role": "string",
              "macAddress": "string",
              "pid": "string",
              "licenseLevel": "string",
              "licenseType": "string",
              "sudiSerialNumber": "string",
              "hardwareVersion": "string",
              "stackNumber": 0,
              "softwareVersion": "string",
              "priority": 0
            }
          ],
          "stackRingProtocol": "string",
          "validLicenseLevels": [
            "string"
          ],
          "totalMemberCount": 0
        },
        "reloadRequested": true,
        "addedOn": 0,
        "siteId": "string",
        "aaaCredentials": {
          "password": "string",
          "username": "string"
        },
        "userMicNumbers": [
          "string"
        ],
        "userSudiSerialNos": [
          "string"
        ],
        "addnMacAddrs": [
          "string"
        ],
        "preWorkflowCliOuputs": [
          {
            "cli": "string",
            "cliOutput": "string"
          }
        ],
        "tags": {},
        "sudiRequired": true,
        "smartAccountId": "string",
        "virtualAccountId": "string",
        "populateInventory": true,
        "siteName": "string",
        "name": "string"
      },
      "systemResetWorkflow": {
        "_id": "string",
        "state": "string",
        "type": "string",
        "description": "string",
        "lastupdateOn": 0,
        "imageId": "string",
        "currTaskIdx": 0,
        "addedOn": 0,
        "tasks": [
          {
            "state": "string",
            "type": "string",
            "currWorkItemIdx": 0,
            "taskSeqNo": 0,
            "endTime": 0,
            "startTime": 0,
            "workItemList": [
              {
                "state": "string",
                "command": "string",
                "outputStr": "string",
                "endTime": 0,
                "startTime": 0,
                "timeTaken": 0
              }
            ],
            "timeTaken": 0,
            "name": "string"
          }
        ],
        "addToInventory": true,
        "instanceType": "string",
        "endTime": 0,
        "execTime": 0,
        "startTime": 0,
        "useState": "string",
        "configId": "string",
        "name": "string",
        "version": 0,
        "tenantId": "string"
      },
      "systemWorkflow": {
        "_id": "string",
        "state": "string",
        "type": "string",
        "description": "string",
        "lastupdateOn": 0,
        "imageId": "string",
        "currTaskIdx": 0,
        "addedOn": 0,
        "tasks": [
          {
            "state": "string",
            "type": "string",
            "currWorkItemIdx": 0,
            "taskSeqNo": 0,
            "endTime": 0,
            "startTime": 0,
            "workItemList": [
              {
                "state": "string",
                "command": "string",
                "outputStr": "string",
                "endTime": 0,
                "startTime": 0,
                "timeTaken": 0
              }
            ],
            "timeTaken": 0,
            "name": "string"
          }
        ],
        "addToInventory": true,
        "instanceType": "string",
        "endTime": 0,
        "execTime": 0,
        "startTime": 0,
        "useState": "string",
        "configId": "string",
        "name": "string",
        "version": 0,
        "tenantId": "string"
      },
      "workflow": {
        "_id": "string",
        "state": "string",
        "type": "string",
        "description": "string",
        "lastupdateOn": 0,
        "imageId": "string",
        "currTaskIdx": 0,
        "addedOn": 0,
        "tasks": [
          {
            "state": "string",
            "type": "string",
            "currWorkItemIdx": 0,
            "taskSeqNo": 0,
            "endTime": 0,
            "startTime": 0,
            "workItemList": [
              {
                "state": "string",
                "command": "string",
                "outputStr": "string",
                "endTime": 0,
                "startTime": 0,
                "timeTaken": 0
              }
            ],
            "timeTaken": 0,
            "name": "string"
          }
        ],
        "addToInventory": true,
        "instanceType": "string",
        "endTime": 0,
        "execTime": 0,
        "startTime": 0,
        "useState": "string",
        "configId": "string",
        "name": "string",
        "version": 0,
        "tenantId": "string"
      },
      "runSummaryList": [
        {
          "details": "string",
          "historyTaskInfo": {
            "type": "string",
            "workItemList": [
              {
                "state": "string",
                "command": "string",
                "outputStr": "string",
                "endTime": 0,
                "startTime": 0,
                "timeTaken": 0
              }
            ],
            "timeTaken": 0,
            "addnDetails": [
              {
                "key": "string",
                "value": "string"
              }
            ],
            "name": "string"
          },
          "errorFlag": true,
          "timestamp": 0
        }
      ],
      "workflowParameters": {
        "topOfStackSerialNumber": "string",
        "licenseLevel": "string",
        "licenseType": "string",
        "configList": [
          {
            "configParameters": [
              {
                "key": "string",
                "value": "string"
              }
            ],
            "configId": "string"
          }
        ]
      },
      "dayZeroConfig": {
        "config": "string"
      },
      "dayZeroConfigPreview": {},
      "version": 0,
      "tenantId": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2020, nkatarmal-crest <nirav.katarmal@crestdatasys.com>
# Copyright: (c) 2020, Cindy Zhao <cizhao@cisco.com>
# Copyright: (c) 2024, Samita Bhattacharjee <samitab@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = r"""
---
module: aci_cloud_subnet
short_description: Manage Cloud Subnet (cloud:Subnet)
description:
- Manage Cloud Subnet on Cisco Cloud ACI.
notes:
- More information about the internal APIC class B(cloud:Subnet) from
  L(the APIC Management Information Model reference,https://developer.cisco.com/docs/apic-mim-ref/).
author:
- Nirav (@nirav)
- Cindy Zhao (@cizhao)
- Samita Bhattacharjee (@samitab)
options:
  name:
    description:
    - The name of the Cloud Subnet.
    type: str
  description:
    description:
    - Description of the Cloud Subnet.
    type: str
  address:
    description:
    - Ip address of the Cloud Subnet.
    type: str
    aliases: [subnet]
  name_alias:
    description:
    - An alias for the name of the current object. This relates to the nameAlias field in ACI and is used to rename object without changing the DN.
    type: str
  tenant:
    description:
    - The name of tenant.
    type: str
    required: true
  cloud_context_profile:
    description:
    - The name of cloud context profile.
    type: str
    required: true
  cidr:
    description:
    - Address of cloud cidr.
    type: str
    required: true
  aws_availability_zone:
    description:
    - The cloud zone which is attached to the given cloud context profile.
    - Only used when it is an AWS Cloud APIC.
    type: str
    aliases: [availability_zone, av_zone, zone]
  vnet_gateway:
    description:
    - Determine if a vNet Gateway Router will be deployed or not.
    - Only used when it is an Azure Cloud APIC.
    type: bool
    default: false
  azure_region:
    description:
    - The Azure cloud region to attach this subnet to.
    - Only used when it is an Azure Cloud APIC.
    type: str
    aliases: [az_region]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    choices: [ absent, present, query ]
    default: present
    type: str
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
"""

EXAMPLES = r"""
- name: Create AWS aci cloud subnet
  cisco.aci.aci_cloud_subnet:
    host: apic
    username: userName
    password: somePassword
    validate_certs: false
    tenant: anstest
    cloud_context_profile: aws_cloudCtxProfile
    cidr: '10.10.0.0/16'
    aws_availability_zone: us-west-1a
    address: 10.10.0.1
  delegate_to: localhost

- name: Create Azure aci cloud subnet
  cisco.aci.aci_cloud_subnet:
    host: apic
    username: userName
    password: somePassword
    validate_certs: false
    tenant: anstest
    cloud_context_profile: azure_cloudCtxProfile
    cidr: '10.10.0.0/16'
    azure_region: westus2
    vnet_gateway: true
    address: 10.10.0.1
  delegate_to: localhost

- name: Query a specific subnet
  cisco.aci.aci_cloud_subnet:
    host: apic
    username: userName
    password: somePassword
    validate_certs: false
    tenant: anstest
    cloud_context_profile: ctx_profile_1
    cidr: '10.10.0.0/16'
    address: 10.10.0.1
    state: query
  delegate_to: localhost

- name: Query all subnets
  cisco.aci.aci_cloud_subnet:
    host: apic
    username: userName
    password: somePassword
    validate_certs: false
    tenant: anstest
    cloud_context_profile: ctx_profile_1
    cidr: '10.10.0.0/16'
    state: query
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec
from ansible.module_utils.basic import AnsibleModule


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        name=dict(type="str", aliases=["subnet"]),
        description=dict(type="str"),
        address=dict(type="str"),
        name_alias=dict(type="str"),
        vnet_gateway=dict(type="bool", default=False),
        tenant=dict(type="str", required=True),
        cloud_context_profile=dict(type="str", required=True),
        cidr=dict(type="str", required=True),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
        aws_availability_zone=dict(type="str", aliases=["availability_zone", "av_zone", "zone"]),
        azure_region=dict(type="str", aliases=["az_region"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["tenant", "cloud_context_profile", "cidr", "address"]],
            ["state", "present", ["tenant", "cloud_context_profile", "cidr", "address"]],
        ],
        mutually_exclusive=[("aws_availability_zone", "azure_region")],
    )

    name = module.params.get("name")
    description = module.params.get("description")
    address = module.params.get("address")
    name_alias = module.params.get("name_alias")
    vnet_gateway = module.params.get("vnet_gateway")
    tenant = module.params.get("tenant")
    cloud_context_profile = module.params.get("cloud_context_profile")
    cidr = module.params.get("cidr")
    state = module.params.get("state")
    aws_availability_zone = module.params.get("aws_availability_zone")
    azure_region = module.params.get("azure_region")
    child_configs = []

    aci = ACIModule(module)
    aci.construct_url(
        root_class=dict(aci_class="fvTenant", aci_rn="tn-{0}".format(tenant), target_filter='eq(fvTenant.name, "{0}")'.format(tenant), module_object=tenant),
        subclass_1=dict(
            aci_class="cloudCtxProfile",
            aci_rn="ctxprofile-{0}".format(cloud_context_profile),
            target_filter='eq(cloudCtxProfile.name, "{0}")'.format(cloud_context_profile),
            module_object=cloud_context_profile,
        ),
        subclass_2=dict(aci_class="cloudCidr", aci_rn="cidr-[{0}]".format(cidr), target_filter='eq(cloudCidr.addr, "{0}")'.format(cidr), module_object=cidr),
        subclass_3=dict(
            aci_class="cloudSubnet", aci_rn="subnet-[{0}]".format(address), target_filter='eq(cloudSubnet.ip, "{0}")'.format(address), module_object=address
        ),
        child_classes=["cloudRsZoneAttach"],
    )

    aci.get_existing()

    if state == "present":
        # in aws cloud apic
        if aws_availability_zone:
            region = aws_availability_zone[:-1]
            tDn = "uni/clouddomp/provp-aws/region-{0}/zone-{1}".format(region, aws_availability_zone)
            child_configs.append({"cloudRsZoneAttach": {"attributes": {"tDn": tDn}}})
        # in azure cloud apic
        if azure_region:
            tDn = "uni/clouddomp/provp-azure/region-{0}/zone-default".format(azure_region)
            child_configs.append({"cloudRsZoneAttach": {"attributes": {"tDn": tDn}}})
        if vnet_gateway:
            usage = "gateway"
        else:
            usage = "user"

        aci.payload(
            aci_class="cloudSubnet",
            class_config=dict(
                name=name,
                descr=description,
                ip=address,
                nameAlias=name_alias,
                scope="private",
                usage=usage,
            ),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="cloudSubnet")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: ipsec_info
short_description: Information module for Ipsec Info
description:
- Get all Ipsec Info.
- Get Ipsec Info by id.
- Returns all the IPsec enabled nodes with configuration details.
- Returns the IPsec configuration details of a given node with the hostname and the NAD IP.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module_info
author: Rafael Campos (@racampos)
options:
  page:
    description:
    - Page query parameter. Page number.
    type: int
  size:
    description:
    - Size query parameter. Number of objects returned per page.
    type: int
  filter:
    description:
    - >
      Filter query parameter. .. Container **Simple filtering** should be available through the filter query
      string parameter. The structure of a filter is a triplet of field operator and value separated with dots.
      More than one filter can be sent. The logical operator common to ALL filter criteria will be by default AND,
      and can be changed by using the *"filterType=or"* query string parameter.
    - Each resource Data model description should specify if an attribute is a filtered field.
    - The 'EQ' operator describes 'Equals'.
    - The 'NEQ' operator describes 'Not Equals'.
    - The 'GT' operator describes 'Greater Than'.
    - The 'LT' operator describes 'Less Than'.
    - The 'STARTSW' operator describes 'Starts With'.
    - The 'NSTARTSW' operator describes 'Not Starts With'.
    - The 'ENDSW' operator describes 'Ends With'.
    - The 'NENDSW' operator describes 'Not Ends With'.
    - The 'CONTAINS' operator describes 'Contains'.
    - The 'NCONTAINS' operator describes 'Not Contains'.
    type: str
  filterType:
    description:
    - >
      FilterType query parameter. The logical operator common to ALL filter criteria will be by default AND, and
      can be changed by using the parameter.
    type: str
  sort:
    description:
    - Sort query parameter. Sort type - asc or desc.
    type: str
  sortBy:
    description:
    - >
      SortBy query parameter. Sort column - The IPsec enabled nodes are sorted based on the columns. This is
      applicable for the field - hostName.
    type: str
  hostName:
    description:
    - HostName path parameter. Hostname of the deployed node.
    type: str
  nadIp:
    description:
    - NadIp path parameter. IP address of the NAD.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    native_ipsec.NativeIpsec.get_ipsec_enabled_nodes_generator,
    native_ipsec.NativeIpsec.get_ipsec_node,

  - Paths used are
    get /api/v1/ipsec,
    get /api/v1/ipsec/{hostName}/{nadIp},

"""

EXAMPLES = r"""
- name: Get all Ipsec Info
  cisco.ise.ipsec_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    page: 0
    size: 0
    filter: string
    filterType: string
    sort: string
    sortBy: string
  register: result

- name: Get Ipsec Info by id
  cisco.ise.ipsec_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    hostName: string
    nadIp: string
  register: result

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "authType": "string",
      "certId": "string",
      "configureVti": true,
      "createTime": "string",
      "espAhProtocol": "string",
      "hostName": "string",
      "id": "string",
      "iface": "string",
      "ikeReAuthTime": 0,
      "ikeVersion": "string",
      "link": {
        "href": "string",
        "rel": "string",
        "type": "string"
      },
      "localInternalIp": "string",
      "modeOption": "string",
      "nadIp": "string",
      "phaseOneDHGroup": "string",
      "phaseOneEncryptionAlgo": "string",
      "phaseOneHashAlgo": "string",
      "phaseOneLifeTime": 0,
      "phaseTwoDHGroup": "string",
      "phaseTwoEncryptionAlgo": "string",
      "phaseTwoHashAlgo": "string",
      "phaseTwoLifeTime": 0,
      "psk": "string",
      "remotePeerInternalIp": "string",
      "status": "string",
      "updateTime": "string"
    }

ise_responses:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  version_added: '1.1.0'
  type: list
  elements: dict
  sample: >
    [
      {
        "authType": "string",
        "certId": "string",
        "configureVti": true,
        "createTime": "string",
        "espAhProtocol": "string",
        "hostName": "string",
        "id": "string",
        "iface": "string",
        "ikeReAuthTime": 0,
        "ikeVersion": "string",
        "link": {
          "href": "string",
          "rel": "string",
          "type": "string"
        },
        "localInternalIp": "string",
        "modeOption": "string",
        "nadIp": "string",
        "phaseOneDHGroup": "string",
        "phaseOneEncryptionAlgo": "string",
        "phaseOneHashAlgo": "string",
        "phaseOneLifeTime": 0,
        "phaseTwoDHGroup": "string",
        "phaseTwoEncryptionAlgo": "string",
        "phaseTwoHashAlgo": "string",
        "phaseTwoLifeTime": 0,
        "psk": "string",
        "remotePeerInternalIp": "string",
        "status": "string",
        "updateTime": "string"
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: clients_trend_analytics_v1
short_description: Resource module for Clients Trend Analytics V1
description:
- Manage operation create of the resource Clients Trend Analytics V1.
- >
   Retrieves the trend analytics of client data for the specified time range. The data will be grouped based on the
   given trend time interval. This API facilitates obtaining consolidated insights into the performance and status of
   the clients over the specified start and end time. For detailed information about the usage of the API, please
   refer to the Open API specification document - https //github.com/cisco-en-programmability/catalyst-center-api-
   specs/blob/main/Assurance/CE_Cat_Center_Org-clients1-1.0.0-resolved.yaml.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  aggregateAttributes:
    description: Clients Trend Analytics's aggregateAttributes.
    elements: dict
    suboptions:
      function:
        description: Function.
        type: str
      name:
        description: Name.
        type: str
    type: list
  attributes:
    description: Attributes.
    elements: str
    type: list
  endTime:
    description: End Time.
    type: int
  filters:
    description: Clients Trend Analytics's filters.
    elements: dict
    suboptions:
      key:
        description: Key.
        type: str
      operator:
        description: Operator.
        type: str
      value:
        description: Value.
        type: int
    type: list
  groupBy:
    description: Group By.
    elements: str
    type: list
  headers:
    description: Additional headers.
    type: dict
  page:
    description: Clients Trend Analytics's page.
    suboptions:
      cursor:
        description: Cursor.
        type: str
      limit:
        description: Limit.
        type: int
      timeSortOrder:
        description: Time Sort Order.
        type: str
    type: dict
  startTime:
    description: Start Time.
    type: int
  trendInterval:
    description: Trend Interval.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Clients RetrievesTheTrendAnalyticsDataRelatedToClientsV1
  description: Complete reference of the RetrievesTheTrendAnalyticsDataRelatedToClientsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!retrieves-the-trend-analytics-data-related-to-clients
notes:
  - SDK Method used are
    clients.Clients.retrieves_the_trend_analytics_data_related_to_clients_v1,

  - Paths used are
    post /dna/data/api/v1/clients/trendAnalytics,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.clients_trend_analytics_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    aggregateAttributes:
    - function: string
      name: string
    attributes:
    - string
    endTime: 0
    filters:
    - key: string
      operator: string
      value: 0
    groupBy:
    - string
    headers: '{{my_headers | from_json}}'
    page:
      cursor: string
      limit: 0
      timeSortOrder: string
    startTime: 0
    trendInterval: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "timestamp": 0,
          "groups": [
            {
              "id": "string",
              "attributes": [
                {
                  "name": "string",
                  "value": 0
                }
              ],
              "aggregateAttributes": [
                {
                  "name": "string",
                  "function": "string",
                  "value": 0
                }
              ]
            }
          ]
        }
      ],
      "page": {
        "limit": 0,
        "cursor": "string",
        "count": 0,
        "timeSortOrder": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2024 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for ios_vrf_global
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: ios_vrf_global
short_description: Resource module to configure global VRF definitions.
description: This module provides declarative management of VRF definitions on Cisco IOS.
version_added: 8.0.0
author: Ruchi Pakhle (@Ruchip16)
notes:
  - Tested against Cisco IOS-XE version 17.3 on CML.
  - This module works with connection C(network_cli).
    See U(https://docs.ansible.com/ansible/latest/network/user_guide/platform_ios.html)
  - The module examples uses callback plugin (stdout_callback = yaml) to generate task
    output in yaml format.
options:
  config:
    description: A dictionary containing device configurations for VRF, including a list of VRF definitions.
    type: dict
    suboptions:
      vrfs:
        description: List of VRF definitions.
        type: list
        elements: dict
        suboptions:
          name:
            description: Name of the VRF.
            type: str
            required: true
          description:
            description: VRF specific description
            type: str
          ipv4:
            description: VRF IPv4 configuration
            type: dict
            suboptions:
              multicast:
                description: IPv4 multicast configuration
                type: dict
                suboptions:
                  multitopology:
                    description: Enable multicast-specific topology
                    type: bool
          ipv6:
            description: VRF IPv6 configuration
            type: dict
            suboptions:
              multicast:
                description: IPv6 multicast configuration
                type: dict
                suboptions:
                  multitopology:
                    description:  Enable multicast-specific topology
                    type: bool
          rd:
            description: Specify route distinguisher (RD).
            type: str
          route_target:
            description: Specify target VPN extended configurations.
            type: dict
            suboptions:
              export:
                description: Export target-VPN configuration.
                type: str
              import_config:
                description: Import target-VPN configuration.
                type: str
              both:
                description: Both export and import target-VPN configuration.
                type: str
          vnet:
            description: Virtual networking configuration.
            type: dict
            suboptions:
              tag:
                description: Identifier used to tag packets associated with this VNET.
                type: int
          vpn:
            description: Configure vpn-id for the VRF as specified in RFC 2685.
            type: dict
            suboptions:
              id:
                description: Configure vpn-id in RFC 2685 format.
                type: str
  running_config:
    description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the IOS device by
        executing the command B(show running-config | section ^vrf).
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
    type: str
  state:
    choices: [parsed, gathered, deleted, merged, replaced, rendered, overridden, purged]
    default: merged
    description:
      - The state the configuration should be left in
      - The states I(rendered), I(gathered) and I(parsed) does not perform any change
        on the device.
      - The state I(rendered) will transform the configuration in C(config) option to
        platform specific CLI commands which will be returned in the I(rendered) key
        within the result. For state I(rendered) active connection to remote host is
        not required.
      - The state I(gathered) will fetch the running configuration from device and transform
        it into structured data in the format as per the resource module argspec and
        the value is returned in the I(gathered) key within the result.
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into JSON format as per the resource module parameters and the
        value is returned in the I(parsed) key within the result. The value of C(running_config)
        option should be the same format as the output of command I(show running-config | section vrf).
        connection to remote host is not required.
      - The state I(deleted) only removes the VRF attributes that this module
        manages and does not negate the VRF completely. Thereby, preserving
        address-family related configurations under VRF context.
      - The state I(purged) removes all the VRF definitions from the
        target device. Use caution with this state.
      - Refer to examples for more details.
    type: str
"""

EXAMPLES = """
# Using merged

# Before state:
# -------------
#
# admin#show running-config | section ^vrf

- name: Merge provided configuration with device configuration
  cisco.ios.ios_vrf_global:
    config:
      vrfs:
        - name: VRF2
          description: This is a test VRF for merged state
          ipv4:
            multicast:
              multitopology: true
          ipv6:
            multicast:
              multitopology: true
          rd: "2:3"
          route_target:
            export: "192.0.2.0:100"
            import_config: "192.0.2.3:200"
          vpn:
            id: "2:45"
          vnet:
            tag: 200
    state: merged

# Task output
# -------------
#
# before: {}
#
# commands:
#   - vrf definition VRF2
#   - description This is a test VRF for merged state
#   - ipv4 multicast multitopology
#   - ipv6 multicast multitopology
#   - rd 2:3
#   - route-target export 192.0.2.0:100
#   - route-target import 192.0.2.3:200
#   - vnet tag 200
#   - vpn id 2:45
#
# after:
#   - name: VRF2
#     description: This is a test VRF for merged state
#     ipv4:
#       multicast:
#         multitopology: true
#     ipv6:
#       multicast:
#         multitopology: true
#     rd: "2:3"
#     route_target:
#       export: "192.0.2.0:100"
#       import_config: "192.0.2.3:200"
#     vnet:
#       tag: 200
#     vpn:
#       id: "2:45"

# After state:
# -------------
#
# admin#show running-config | section ^vrf
# vrf definition VRF2
#  vnet tag 200
#  description This is a test VRF for merged state
#  ipv4 multicast multitopology
#  ipv6 multicast multitopology
#  rd 2:3
#  vpn id 2:45
#  route-target export 192.0.2.0:100
#  route-target import 192.0.2.3:200

# Using replaced

# Before state:
# -------------
#
# admin#show running-config | section ^vrf
# vrf definition VRF2
#  vnet tag 200
#  description This is a test VRF for merged state
#  ipv4 multicast multitopology
#  ipv6 multicast multitopology
#  rd 2:3
#  vpn id 2:45
#  route-target export 192.0.2.0:100
#  route-target import 192.0.2.3:200

- name: Replace the provided configuration with the existing running configuration
  cisco.ios.ios_vrf_global:
    config:
      vrfs:
        - name: VRF7
          description: VRF7 description
          ipv4:
            multicast:
              multitopology: true
          ipv6:
            multicast:
              multitopology: true
          rd: "7:8"
          route_target:
            export: "198.51.100.112:500"
            import_config: "192.0.2.4:400"
          vpn:
            id: "5:45"
          vnet:
            tag: 300
    state: replaced

# Task Output:
# ------------
#
# before:
#   - name: VRF2
#     description: This is a test VRF for merged state
#     ipv4:
#       multicast:
#         multitopology: true
#     ipv6:
#       multicast:
#         multitopology: true
#     rd: "2:3"
#     route_target:
#       export: "192.0.2.0:100"
#       import_config: "192.0.2.3:200"
#     vnet:
#       tag: 200
#     vpn:
#       id: "2:45"
#
# commands:
# - vrf definition VRF7
# - description VRF7 description
# - ipv4 multicast multitopology
# - ipv6 multicast multitopology
# - rd 7:8
# - route-target export 198.51.100.112:500
# - route-target import 192.0.2.4:400
# - vnet tag 300
# - vpn id 5:45
#
# after:
#   - name: VRF2
#     description: This is a test VRF for merged state
#     ipv4:
#       multicast:
#         multitopology: true
#     ipv6:
#       multicast:
#         multitopology: true
#     rd: "2:3"
#     route_target:
#       export: "192.0.2.0:100"
#       import_config: "192.0.2.3:200"
#     vnet:
#       tag: 200
#     vpn:
#       id: "2:45
#   - name: VRF7
#     description: VRF7 description
#     ipv4:
#       multicast:
#         multitopology: true
#     ipv6:
#       multicast:
#         multitopology: true
#     rd: "7:8"
#     route_target:
#       export: "198.51.100.112:500"
#       import_config: "192.0.2.4:400"
#     vnet:
#       tag: 300
#     vpn:
#       id: "5:45"
#
# After state:
# -------------
#
# admin#show running-config | section ^vrf
# vrf definition VRF2
#  vnet tag 200
#  description This is a test VRF for merged state
#  ipv4 multicast multitopology
#  ipv6 multicast multitopology
#  rd 2:3
#  vpn id 2:45
#  route-target export 192.0.2.0:100
#  route-target import 192.0.2.3:200
# vrf definition VRF7
#  vnet tag 300
#  description VRF7 description
#  ipv4 multicast multitopology
#  ipv6 multicast multitopology
#  rd 7:8
#  route-target export 198.51.100.112:500
#  route-target import 192.0.2.4:400
#  vpn id 5:45

# Using Overridden

# Before state:
# -------------
#
# admin#show running-config | section ^vrf
# vrf definition VRF2
#  vnet tag 200
#  description This is a test VRF for merged state
#  ipv4 multicast multitopology
#  ipv6 multicast multitopology
#  rd 2:3
#  vpn id 2:45
#  route-target export 192.0.2.0:100
#  route-target import 192.0.2.3:200
# vrf definition VRF7
#  vnet tag 300
#  description VRF7 description
#  ipv4 multicast multitopology
#  ipv6 multicast multitopology
#  rd 7:8
#  route-target export 198.51.100.112:500
#  route-target import 192.0.2.4:400
#  vpn id 5:45

- name: Override the provided configuration with the existing running configuration
  cisco.ios.ios_vrf_global:
    config:
      vrfs:
        - name: VRF6
          description: VRF6 description
          ipv4:
            multicast:
              multitopology: true
          ipv6:
            multicast:
              multitopology: true
          rd: "6:7"
          route_target:
            export: "198.51.0.2:400"
            import_config: "198.51.0.5:200"
          vpn:
            id: "4:5"
          vnet:
            tag: 500
    state: overridden

# Task Output:
# ------------
#
# before:
#   - name: VRF2
#     description: This is a test VRF for merged state
#     ipv4:
#       multicast:
#         multitopology: true
#     ipv6:
#       multicast:
#         multitopology: true
#     rd: "2:3"
#     route_target:
#       export: "192.0.2.0:100"
#       import_config: "192.0.2.3:200"
#     vnet:
#       tag: 200
#     vpn:
#       id: "2:45
#   - name: VRF7
#     description: VRF7 description
#     ipv4:
#       multicast:
#         multitopology: true
#     ipv6:
#       multicast:
#         multitopology: true
#     rd: "7:8"
#     route_target:
#       export: "198.51.100.112:500"
#       import_config: "192.0.2.4:400"
#     vnet:
#       tag: 300
#     vpn:
#       id: "5:45"
#
# commands:
# - vrf definition VRF2
# - no description This is a test VRF for merged state
# - no ipv4 multicast multitopology
# - no ipv6 multicast multitopology
# - no rd 2:3
# - no route-target export 192.0.2.0:100
# - no route-target import 192.0.2.3:200
# - no vnet tag 200
# - no vpn id 2:45
# - vrf definition VRF7
# - no description VRF7 description
# - no ipv4 multicast multitopology
# - no ipv6 multicast multitopology
# - no rd 7:8
# - no route-target export 198.51.100.112:500
# - no route-target import 192.0.2.4:400
# - no vnet tag 300
# - no vpn id 5:45
# - vrf definition VRF6
# - description VRF6 description
# - ipv4 multicast multitopology
# - ipv6 multicast multitopology
# - rd 6:7
# - route-target export 198.51.0.2:400
# - route-target import 198.51.0.5:200
# - vnet tag 500
# - vpn id 4:5
#
# after:
#   - name: VRF2
#   - name: VRF6
#     description: VRF6 description
#     ipv4:
#       multicast:
#         multitopology: true
#     ipv6:
#       multicast:
#         multitopology: true
#     rd: "6:7"
#     route_target:
#       export: "198.51.0.2:400"
#       import_config: "198.51.0.5:200"
#     vnet:
#       tag: 500
#     vpn:
#       id: "4:5
#   - name: VRF7

# After state:
# ------------
#
# admin#show running-config | section ^vrf
# vrf definition VRF2
# vrf definition VRF6
#  vnet tag 500
#  description VRF6 description
#  ipv4 multicast multitopology
#  ipv6 multicast multitopology
#  rd 6:7
#  vpn id 4:5
#  route-target export 198.51.0.2:400
#  route-target import 198.51.0.5:200
# vrf definition VRF7

# Using Deleted

# Before state:
# -------------
#
# admin#show running-config | section ^vrf
# vrf definition VRF2
# vrf definition VRF6
#  vnet tag 500
#  description VRF6 description
#  ipv4 multicast multitopology
#  ipv6 multicast multitopology
#  rd 6:7
#  vpn id 4:5
#  route-target export 198.51.0.2:400
#  route-target import 198.51.0.5:200
# vrf definition VRF7

- name: Delete the provided configuration when config is given
  cisco.ios.ios_vrf_global:
    config:
      vrfs:
        - name: VRF2
        - name: VRF6
        - name: VRF7
    state: deleted

# Task Output:
# ------------
#
# before:
#   - name: VRF2
#   - name: VRF6
#     description: VRF6 description
#     ipv4:
#       multicast:
#         multitopology: true
#     ipv6:
#       multicast:
#         multitopology: true
#     rd: "6:7"
#     route_target:
#       export: "198.51.0.2:400"
#       import_config: "198.51.0.5:200"
#     vnet:
#       tag: 500
#     vpn:
#       id: "4:5"
#   - name: VRF7
#
# commands:
# - vrf definition VRF2
# - vrf definition VRF6
# - no description VRF6 description
# - no ipv4 multicast multitopology
# - no ipv6 multicast multitopology
# - no rd 6:7
# - no route-target export 198.51.0.2:400
# - no route-target import 198.51.0.5:200
# - no vnet tag 500
# - no vpn id 4:5
# - vrf definition VRF7
#
# after:
#   - name: VRF2
#   - name: VRF6
#   - name: VRF7

# After state:
# -------------
# admin#show running-config | section ^vrf
# vrf definition VRF2
# vrf definition VRF6
# vrf definition VRF7

# Using Deleted with empty config

# Before state:
# -------------
#
# admin#show running-config | section ^vrf
# vrf definition VRF2
# vrf definition VRF6
#  vnet tag 500
#  description VRF6 description
#  ipv4 multicast multitopology
#  ipv6 multicast multitopology
#  rd 6:7
#  vpn id 4:5
#  route-target export 198.51.0.2:400
#  route-target import 198.51.0.5:200
# vrf definition VRF7

- name: Delete the provided configuration when config is empty
  cisco.ios.ios_vrf_global:
    config:
    state: deleted

# Task Output:
# ------------
#
# before:
#   - name: VRF2
#   - name: VRF6
#     description: VRF6 description
#     ipv4:
#       multicast:
#         multitopology: true
#     ipv6:
#       multicast:
#         multitopology: true
#     rd: "6:7"
#     route_target:
#       export: "198.51.0.2:400"
#       import_config: "198.51.0.5:200"
#     vnet:
#       tag: 500
#     vpn:
#       id: "4:5"
#   - name: VRF7

# commands:
# - vrf definition VRF2
# - vrf definition VRF6
# - no description VRF6 description
# - no ipv4 multicast multitopology
# - no ipv6 multicast multitopology
# - no rd 6:7
# - no route-target export 198.51.0.2:400
# - no route-target import 198.51.0.5:200
# - no vnet tag 500
# - no vpn id 4:5
# - vrf definition VRF7
#
# after:
#   - name: VRF2
#   - name: VRF6
#   - name: VRF7

# After state:
# -------------
#
# admin#show running-config | section ^vrf
# vrf definition VRF2
# vrf definition VRF6
# vrf definition VRF7

# Using purged - would delete all the VRF definitions

# Before state:
# -------------
#
# admin#show running-config | section ^vrf
# vrf definition VRF2
# vrf definition VRF6
# vrf definition VRF7

- name: Purge all the configuration from the device
  cisco.ios.ios_vrf_global:
    state: purged

# Task Output:
# ------------
#
# before:
#   - name: VRF2
#   - name: VRF6
#   - name: VRF7
# commands:
# - no vrf definition VRF2
# - no vrf definition VRF6
# - no vrf definition VRF7
# after: {}

# After state:
# -------------
#
# admin#show running-config | section ^vrf

# Using Rendered

- name: Render provided configuration with device configuration
  cisco.ios.ios_vrf_global:
    config:
      vrfs:
        - name: VRF2
          description: This is a test VRF for merged state
          ipv4:
            multicast:
              multitopology: true
          ipv6:
            multicast:
              multitopology: true
          rd: "2:3"
          route_target:
            export: "192.0.2.0:100"
            import_config: "192.0.2.3:200"
          vpn:
            id: "2:45"
          vnet:
            tag: 200
    state: rendered

# Task Output:
# ------------
#
# rendered:
# - vrf definition VRF2
# - description This is a test VRF for merged state
# - ipv4 multicast multitopology
# - ipv6 multicast multitopology
# - rd 2:3
# - route-target export 192.0.2.0:100
# - route-target import 192.0.2.3:200
# - vnet tag 200
# - vpn id 2:45

# Using Gathered

# Before state:
# -------------
#
# admin#show running-config | section ^vrf
# vrf definition VRF2
#  vnet tag 200
#  description This is a test VRF for merged state
#  ipv4 multicast multitopology
#  ipv6 multicast multitopology
#  rd 2:3
#  vpn id 2:45
#  route-target export 192.0.2.0:100
#  route-target import 192.0.2.3:200

- name: Gather existing running configuration
  cisco.ios.ios_vrf_global:
    config:
    state: gathered

# Task Output:
# ------------
#
# gathered:
#   vrfs:
#     - name: VRF2
#       description: This is a test VRF for merged state
#       ipv4:
#         multicast:
#           multitopology: true
#       ipv6:
#         multicast:
#           multitopology: true
#       rd: "2:3"
#       route_target:
#         export: "192.0.2.0:100"
#         import_config: "192.0.2.3:200"
#       vnet:
#         tag: 200
#       vpn:
#         id: "2:45"

# Using parsed

# File: parsed.cfg
# ----------------
#
# vrf definition test
#  vnet tag 34
#  description This is test VRF
#  ipv4 multicast multitopology
#  ipv6 multicast multitopology
#  rd 192.0.2.0:300
#  vpn id 3:4
#  route-target export 192.0.2.0:100
#  route-target import 192.0.2.2:300
# vrf definition test2
#  vnet tag 35
#  description This is test VRF
#  ipv4 multicast multitopology
#  ipv6 multicast multitopology
#  rd 192.0.2.3:300

- name: Parse the provided configuration
  cisco.ios.ios_vrf_global:
    running_config: "{{ lookup('file', 'parsed.cfg') }}"
    state: parsed

# Task Output:
# ------------
#
# parsed:
#   vrfs:
#     - name: test
#       description: This is test VRF
#       ipv4:
#         multicast:
#           multitopology: true
#       ipv6:
#         multicast:
#           multitopology: true
#       rd: "192.0.2.0:300"
#       route_target:
#         export: "192.0.2.0:100"
#         import_config: "192.0.2.2:300"
#       vnet:
#         tag: 34
#       vpn:
#         id: "3:4"
#     - name: test2
#       description: This is test VRF
#       ipv4:
#         multicast:
#           multitopology: true
#       ipv6:
#         multicast:
#           multitopology: true
#       rd: "192.0.2.3:300"
#       vnet:
#         tag: 35
"""

RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
commands:
  description: The set of commands pushed to the remote device.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: list
  sample:
    - "vrf definition test"
    - "description This is a test VRF"
    - "rd: 2:3"
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
    - "vrf definition management"
    - "description This is a test VRF"
    - "rd: 2:3"
    - "route-target export 190.0.2.3:400"
    - "route-target import 190.0.2.1:300"
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
"""


from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.argspec.vrf_global.vrf_global import (
    Vrf_globalArgs,
)
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.config.vrf_global.vrf_global import (
    Vrf_global,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Vrf_globalArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Vrf_global(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: user_equipment_bulk
short_description: Resource module for User Equipment Bulk
description:
- Manage operation create of the resource User Equipment Bulk.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  ItemList:
    description: User Equipment Bulk's ItemList.
    elements: dict
    suboptions:
      description:
        description: User Equipment Bulk's description.
        type: str
      deviceGroup:
        description: Device or Endpoint Group.
        type: str
      id:
        description: User Equipment Bulk's id.
        type: str
      imei:
        description: User Equipment Bulk's imei.
        type: str
    type: list
  operation:
    description: User Equipment Bulk's operation.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    user_equipment.UserEquipment.bulk_user_equipment_operation,

  - Paths used are
    post /api/v1/fiveg/user-equipment/bulk,

"""

EXAMPLES = r"""
- name: Create
  cisco.ise.user_equipment_bulk:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    ItemList:
    - description: string
      deviceGroup: string
      id: string
      imei: string
    operation: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: event_artifact_info
short_description: Information module for Event Artifact Info
description:
- This module represents an alias of the module event_artifact_v1_info
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  eventIds:
    description:
    - EventIds query parameter. List of eventIds.
    type: str
  tags:
    description:
    - Tags query parameter. Tags defined.
    type: str
  offset:
    description:
    - Offset query parameter. Record start offset.
    type: float
  limit:
    description:
    - Limit query parameter. # of records to return in result set.
    type: float
  sortBy:
    description:
    - SortBy query parameter. Sort by field.
    type: str
  order:
    description:
    - Order query parameter. Sorting order (asc/desc).
    type: str
  search:
    description:
    - Search query parameter. Findd matches in name, description, eventId, type, category.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Event Management GetEventArtifactsV1
  description: Complete reference of the GetEventArtifactsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-event-artifacts
notes:
  - SDK Method used are
    event_management.EventManagement.get_eventartifacts,

  - Paths used are
    get /dna/system/api/v1/event/artifact,
  - It should be noted that this module is an alias of event_artifact_v1_info

"""

EXAMPLES = r"""
- name: Get all Event Artifact Info
  cisco.dnac.event_artifact_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    eventIds: string
    tags: string
    offset: 0
    limit: 0
    sortBy: string
    order: string
    search: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "version": "string",
        "artifactId": "string",
        "namespace": "string",
        "name": "string",
        "description": "string",
        "domain": "string",
        "subDomain": "string",
        "deprecationMessage": "string",
        "deprecated": true,
        "tags": [
          "string"
        ],
        "isTemplateEnabled": true,
        "ciscoDNAEventLink": "string",
        "note": "string",
        "isPrivate": true,
        "eventPayload": {
          "eventId": "string",
          "version": "string",
          "category": "string",
          "type": "string",
          "source": "string",
          "severity": "string",
          "details": {
            "device_ip": "string",
            "message": "string"
          },
          "additionalDetails": {}
        },
        "eventTemplates": [
          {}
        ],
        "isTenantAware": true,
        "supportedConnectorTypes": [
          "string"
        ],
        "tenantId": "string"
      }
    ]
"""



#
# -*- coding: utf-8 -*-
# Copyright 2023 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The ios_vxlan_vtep config file.
It is in this file where the current configuration (as dict)
is compared to the provided configuration (as dict) and the command set
necessary to bring the current configuration to its desired end-state is
created.
"""

from ansible.module_utils.six import iteritems
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.resource_module import (
    ResourceModule,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    dict_merge,
    param_list_to_dict,
)

from ansible_collections.cisco.ios.plugins.module_utils.network.ios.facts.facts import Facts
from ansible_collections.cisco.ios.plugins.module_utils.network.ios.rm_templates.vxlan_vtep import (
    Vxlan_vtepTemplate,
)


class Vxlan_vtep(ResourceModule):
    """
    The ios_vxlan_vtep config class
    """

    def __init__(self, module):
        super(Vxlan_vtep, self).__init__(
            empty_fact_val={},
            facts_module=Facts(module),
            module=module,
            resource="vxlan_vtep",
            tmplt=Vxlan_vtepTemplate(),
        )
        self.parsers = [
            "source_interface",
            "host_reachability_bgp",
        ]

    def execute_module(self):
        """Execute the module

        :rtype: A dictionary
        :returns: The result from module execution
        """

        if self.state not in ["parsed", "gathered"]:
            self.generate_commands()
            self.run_commands()
        return self.result

    def generate_commands(self):
        """Generate configuration commands to send based on
        want, have and desired state.
        """

        wantd, haved = self._interface_list_to_dict(self.want, self.have)

        # if state is merged, merge want onto have and then compare
        if self.state == "merged":
            wantd = dict_merge(haved, wantd)

        # if state is deleted, empty out wantd and set haved to wantd
        if self.state == "deleted":
            haved = {k: v for k, v in iteritems(haved) if k in wantd or not wantd}
            wantd_copy = wantd.copy()
            wantd = {}

        # remove superfluous config for deleted
        if self.state in ["overridden", "deleted"]:
            for k, have in iteritems(haved):
                if k not in wantd:
                    have = self._filtered_dict(wantd_copy.get(k), have)
                    self._compare(want={}, have=have)

        for k, want in iteritems(wantd):
            self._compare(want=want, have=haved.pop(k, {}))

    def _compare(self, want, have):
        """Leverages the base class `compare()` method and
        populates the list of commands to be run by comparing
        the `want` and `have` data with the `parsers` defined
        for the Vxlan_vtep network resource.
        """

        begin = len(self.commands)
        self.compare(parsers=self.parsers, want=want, have=have)

        self._compare_member_vnis(
            want.pop("member", {}).get("vni", {}),
            have.pop("member", {}).get("vni", {}),
        )

        if len(self.commands) != begin:
            self.commands.insert(
                begin,
                self._tmplt.render(want or have, "interface", False),
            )

    def _compare_member_vnis(self, wantmv, havemv):
        """Compare member VNIs dict"""

        PARSER_DICT = {
            "l2vni": "replication",
            "l3vni": "vrf",
        }

        for vni_type in ["l2vni", "l3vni"]:
            wantd = wantmv.get(vni_type, {})
            haved = havemv.get(vni_type, {})
            undel_vnis = haved.copy()

            for wvni, want in wantd.items():
                have = haved.pop(wvni, {})
                if want != have:
                    # remove exiting config of the member VNI
                    self.addcmd(undel_vnis.pop(wvni, {}), PARSER_DICT[vni_type], True)
                    if vni_type == "l3vni":
                        undel_vnis = self._remove_existing_vnis_vrfs(want["vrf"], undel_vnis)
                    self.addcmd(want, PARSER_DICT[vni_type])

            # remove remaining configs in have for replaced state
            for hvni, have in haved.items():
                if hvni in undel_vnis:
                    self.addcmd(have, PARSER_DICT[vni_type], True)

    def _interface_list_to_dict(self, want, have):
        """Convert all list of dicts to dicts of dicts"""

        wantd = {entry["interface"]: entry for entry in want}
        haved = {entry["interface"]: entry for entry in have}

        for each in wantd, haved:
            if each:
                for nvi, nvid in each.items():
                    member_vni = nvid.get("member", {}).get("vni")
                    if member_vni:
                        for vni_type in member_vni:
                            member_vni[vni_type] = param_list_to_dict(
                                member_vni[vni_type],
                                unique_key="vni",
                                remove_key=False,
                            )

        return wantd, haved

    def _remove_existing_vnis_vrfs(self, want_vrf, haved):
        """Remove member VNIs of corresponding VRF"""

        vrf_haved = next(
            (h for h in haved.values() if h["vrf"] == want_vrf),
            None,
        )
        if vrf_haved:
            self.addcmd(haved.pop(vrf_haved["vni"]), "vrf", True)
        return haved

    def _filtered_dict(self, want, have):
        """Remove other config from 'have' if 'member' key is present"""

        if "member" in want:
            have_member = {}
            want_vni_dict = want.get("member", {}).get("vni", {})
            have_vni_dict = have.get("member", {}).get("vni", {})

            for vni_type, have_vnis in have_vni_dict.items():
                want_vnis = want_vni_dict.get(vni_type, {})
                have_member[vni_type] = {
                    vni: have_vni_dict[vni_type].get(vni) for vni in have_vnis if vni in want_vnis
                }
            have = {
                "interface": have["interface"],
                "member": {"vni": have_member},
            }

        return have



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: custom_attributes
short_description: Resource module for Custom Attributes
description:
- Manage operations create and delete of the resource Custom Attributes.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  attributeName:
    description: Custom Attributes's attributeName.
    type: str
  attributeType:
    description: Custom Attributes's attributeType.
    type: str
  name:
    description: Name path parameter. The name of the custom attribute.
    type: str
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
notes:
  - SDK Method used are
    customattributes.Customattributes.create_custom_attribute,
    customattributes.Customattributes.delete,

  - Paths used are
    post /api/v1/endpoint-custom-attribute,
    delete /api/v1/endpoint-custom-attribute/{name},

"""

EXAMPLES = r"""
- name: Create
  cisco.ise.custom_attributes:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: present
    attributeName: string
    attributeType: string

- name: Delete by name
  cisco.ise.custom_attributes:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    state: absent
    name: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "attributeName": "string",
      "attributeType": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: pnp_global_settings_v1_info
short_description: Information module for Pnp Global Settings V1
description:
- Get all Pnp Global Settings V1.
- Returns global PnP settings of the user.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Device Onboarding (PnP) GetPnPGlobalSettingsV1
  description: Complete reference of the GetPnPGlobalSettingsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-pn-p-global-settings
notes:
  - SDK Method used are
    device_onboarding_pnp.DeviceOnboardingPnp.get_pnp_global_settings_v1,

  - Paths used are
    get /dna/intent/api/v1/onboarding/pnp-settings,

"""

EXAMPLES = r"""
- name: Get all Pnp Global Settings V1
  cisco.dnac.pnp_global_settings_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "savaMappingList": [
        {
          "syncStatus": "string",
          "syncStartTime": 0,
          "syncResult": {
            "syncList": [
              {
                "syncType": "string",
                "deviceSnList": [
                  "string"
                ]
              }
            ],
            "syncMsg": "string"
          },
          "lastSync": 0,
          "tenantId": "string",
          "profile": {
            "port": 0,
            "addressIpV4": "string",
            "addressFqdn": "string",
            "profileId": "string",
            "proxy": true,
            "makeDefault": true,
            "cert": "string",
            "name": "string"
          },
          "token": "string",
          "expiry": 0,
          "ccoUser": "string",
          "smartAccountId": "string",
          "virtualAccountId": "string",
          "autoSyncPeriod": 0,
          "syncResultStr": "string"
        }
      ],
      "taskTimeOuts": {
        "imageDownloadTimeOut": 0,
        "configTimeOut": 0,
        "generalTimeOut": 0
      },
      "tenantId": "string",
      "aaaCredentials": {
        "password": "string",
        "username": "string"
      },
      "defaultProfile": {
        "fqdnAddresses": [
          "string"
        ],
        "proxy": true,
        "cert": "string",
        "ipAddresses": [
          "string"
        ],
        "port": 0
      },
      "acceptEula": true,
      "id": "string",
      "version": 0
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2017, Red Hat, Inc.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r'''
---
module: win_dns_client
short_description: Configures DNS lookup on Windows hosts
description:
  - The M(ansible.windows.win_dns_client) module configures the DNS client on Windows network adapters.
options:
  adapter_names:
    description:
      - Adapter name or list of adapter names for which to manage DNS settings ('*' is supported as a wildcard value).
      - The adapter name used is the connection caption in the Network Control Panel or the InterfaceAlias of C(Get-DnsClientServerAddress).
    type: list
    elements: str
    required: yes
  dns_servers:
    description:
      - Single or ordered list of DNS servers (IPv4 and IPv6 addresses) to configure for lookup.
      - An empty list will configure the adapter to use the DHCP-assigned values on connections where DHCP is enabled,
        or disable DNS lookup on statically-configured connections.
      - IPv6 DNS servers can only be set on Windows Server 2012 or newer, older hosts can only set IPv4 addresses.
    type: list
    elements: str
    required: yes
    aliases: [ "ipv4_addresses", "ip_addresses", "addresses" ]
author:
- Matt Davis (@nitzmahone)
- Brian Scholer (@briantist)
'''

EXAMPLES = r'''
- name: Set a single address on the adapter named Ethernet
  ansible.windows.win_dns_client:
    adapter_names: Ethernet
    dns_servers: 192.168.34.5

- name: Set multiple lookup addresses on all visible adapters (usually physical adapters that are in the Up state), with debug logging to a file
  ansible.windows.win_dns_client:
    adapter_names: '*'
    dns_servers:
      - 192.168.34.5
      - 192.168.34.6
    log_path: C:\dns_log.txt

- name: Set IPv6 DNS servers on the adapter named Ethernet
  ansible.windows.win_dns_client:
    adapter_names: Ethernet
    dns_servers:
      - '2001:db8::2'
      - '2001:db8::3'

- name: Configure all adapters whose names begin with Ethernet to use DHCP-assigned DNS values
  ansible.windows.win_dns_client:
    adapter_names: 'Ethernet*'
    dns_servers: []
'''

RETURN = r'''
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: assurance_events_count_v1_info
short_description: Information module for Assurance Events Count V1
description:
- Get all Assurance Events Count V1.
- >
   API to fetch the count of assurance events that match the filter criteria. Please refer to the 'API Support
   Documentation' section to understand which fields are supported. For detailed information about the usage of the
   API, please refer to the Open API specification document - https //github.com/cisco-en-programmability/catalyst-
   center-api-specs/blob/main/Assurance/CE_Cat_Center_Org-AssuranceEvents-1.0.0-resolved.yaml.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  deviceFamily:
    description:
    - >
      DeviceFamily query parameter. Device family. Please note that multiple families across network device type
      and client type is not allowed. For example, choosing `Routers` along with `Wireless Client` or `Unified AP`
      is not supported. Examples `deviceFamily=Switches and Hubs` (single deviceFamily requested)
      `deviceFamily=Switches and Hubs&deviceFamily=Routers` (multiple deviceFamily requested).
    type: str
  startTime:
    description:
    - >
      StartTime query parameter. Start time from which API queries the data set related to the resource. It must
      be specified in UNIX epochtime in milliseconds. Value is inclusive. If `startTime` is not provided, API will
      default to current time minus 24 hours.
    type: str
  endTime:
    description:
    - >
      EndTime query parameter. End time to which API queries the data set related to the resource. It must be
      specified in UNIX epochtime in milliseconds. Value is inclusive. If `endTime` is not provided, API will
      default to current time.
    type: str
  messageType:
    description:
    - >
      MessageType query parameter. Message type for the event. Examples `messageType=Syslog` (single messageType
      requested) `messageType=Trap&messageType=Syslog` (multiple messageType requested).
    type: str
  severity:
    description:
    - >
      Severity query parameter. Severity of the event between 0 and 6. This is applicable only for events related
      to network devices (other than AP) and `Wired Client` events. | Value | Severity | | ----- | ----------- | |
      0 | Emergency | | 1 | Alert | | 2 | Critical | | 3 | Error | | 4 | Warning | | 5 | Notice | | 6 | Info |
      Examples `severity=0` (single severity requested) `severity=0&severity=1` (multiple severity requested).
    type: str
  siteId:
    description:
    - >
      SiteId query parameter. The UUID of the site. (Ex. `flooruuid`) Examples `?siteId=id1` (single siteId
      requested) `?siteId=id1&siteId=id2&siteId=id3` (multiple siteId requested).
    type: str
  siteHierarchyId:
    description:
    - >
      SiteHierarchyId query parameter. The full hierarchy breakdown of the site tree in id form starting from
      Global site UUID and ending with the specific site UUID. (Ex. `globalUuid/areaUuid/buildingUuid/floorUuid`)
      This field supports wildcard asterisk (`*`) character search support. E.g. `*uuid*, *uuid, uuid*` Examples
      `?siteHierarchyId=globalUuid/areaUuid/buildingUuid/floorUuid `(single siteHierarchyId requested) `?siteHiera
      rchyId=globalUuid/areaUuid/buildingUuid/floorUuid&siteHierarchyId=globalUuid/areaUuid2/buildingUuid2/floorUu
      id2` (multiple siteHierarchyId requested).
    type: str
  networkDeviceName:
    description:
    - >
      NetworkDeviceName query parameter. Network device name. This parameter is applicable for network device
      related families. This field supports wildcard (`*`) character-based search. Ex `*Branch*` or `Branch*` or
      `*Branch` Examples `networkDeviceName=Branch-3-Gateway` (single networkDeviceName requested)
      `networkDeviceName=Branch-3-Gateway&networkDeviceName=Branch-3-Switch` (multiple networkDeviceName
      requested).
    type: str
  networkDeviceId:
    description:
    - >
      NetworkDeviceId query parameter. The list of Network Device Uuids. (Ex.
      `6bef213c-19ca-4170-8375-b694e251101c`) Examples `networkDeviceId=6bef213c-19ca-4170-8375-b694e251101c`
      (single networkDeviceId requested) `networkDeviceId=6bef213c-19ca-4170-8375-
      b694e251101c&networkDeviceId=32219612-819e-4b5e-a96b-cf22aca13dd9&networkDeviceId=2541e9a7-b80d-4955-8aa2-
      79b233318ba0` (multiple networkDeviceId requested).
    type: str
  apMac:
    description:
    - >
      ApMac query parameter. MAC address of the access point. This parameter is applicable for `Unified AP` and
      `Wireless Client` events. This field supports wildcard (`*`) character-based search. Ex `*50 0F*` or `50
      0F*` or `*50 0F` Examples `apMac=50 0F 80 0F F7 E0` (single apMac requested) `apMac=50 0F 80 0F F7
      E0&apMac=18 80 90 AB 7E A0` (multiple apMac requested).
    type: str
  clientMac:
    description:
    - >
      ClientMac query parameter. MAC address of the client. This parameter is applicable for `Wired Client` and
      `Wireless Client` events. This field supports wildcard (`*`) character-based search. Ex `*66 2B*` or `66
      2B*` or `*66 2B` Examples `clientMac=66 2B B8 D2 01 56` (single clientMac requested) `clientMac=66 2B B8 D2
      01 56&clientMac=DC A6 32 F5 5A 89` (multiple clientMac requested).
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices CountTheNumberOfEventsV1
  description: Complete reference of the CountTheNumberOfEventsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!count-the-number-of-events
notes:
  - SDK Method used are
    devices.Devices.count_the_number_of_events_v1,

  - Paths used are
    get /dna/data/api/v1/assuranceEvents/count,

"""

EXAMPLES = r"""
- name: Get all Assurance Events Count V1
  cisco.dnac.assurance_events_count_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    deviceFamily: string
    startTime: string
    endTime: string
    messageType: string
    severity: string
    siteId: string
    siteHierarchyId: string
    networkDeviceName: string
    networkDeviceId: string
    apMac: string
    clientMac: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "count": 0
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright (c) 2024, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

"""Ansible module to manage Extranet Policy Operations in SD-Access Fabric in Cisco Catalyst Center."""
from __future__ import absolute_import, division, print_function

__metaclass__ = type
__author__ = ("Rugvedi Kapse, Madhan Sankaranarayanan")

DOCUMENTATION = r"""
---
module: sda_extranet_policies_workflow_manager
short_description: SDA Extranet Policies Module provides functionality for managing SD-Access Extranet Policies in Cisco Catalyst Center.
description:
  - Manage SD-Access Extranet Policy operations such as create, update, or delete extranet policies in Cisco Catalyst Center.
  - API to create a new extranet policy.
  - API to update an existing or edit an existing extranet policy.
  - API for deletion of an existing extranet policy using the policy name.
version_added: "6.17.0"
extends_documentation_fragment:
  - cisco.dnac.workflow_manager_params
author: Rugvedi Kapse (@rukapse)
        Madhan Sankaranarayanan (@madhansansel)
options:
  config_verify:
    description: Set to True to verify the Cisco Catalyst Center config after applying the playbook config.
    type: bool
    default: False
  state:
    description: State of Cisco Catalyst Center after module completion.
    type: str
    choices: [ merged, deleted ]
    default: merged
  config:
    description: List of Extranet Policy Details for Creating, Updating, or Deleting Operations.
    type: list
    elements: dict
    required: True
    suboptions:
      extranet_policy_name:
        description:
            - Name of the SDA Extranet Policy.
            - Used to create, update, or delete the policy.
            - Required for all operations (create, update, delete).
            - Cannot be modified once set.
        type: str
      provider_virtual_network:
        description:
            - Specifies the Provider Virtual Network containing shared services resources that subscribers need to access.
            - If a virtual network is already defined as a Provider, it cannot be assigned as a provider again.
            - Ensure the default route is present in the Global Routing Table if INFRA_VN is defined as the Provider.
            - For Subscriber Virtual Networks with multiple Providers having overlapping routes, traffic will be
              load-balanced across those Provider Virtual Networks.
            - Required for creating or updating the policy.
            - Updating this field is not allowed.
        type: str
      subscriber_virtual_networks:
        description:
            - Specifies a list of Subscriber Virtual Networks that require access to the Provider Virtual Network
              containing shared services resources.
            - A Virtual Network previously defined as a Provider cannot be selected as a subscriber.
            - Required for creating or updating the policy.
            - Can be modified.
            - Example - ["VN_2", "VN_4"]
        type: list
        elements: str
      fabric_sites:
        description:
            - Specifies the Fabric Site(s) where this Extranet Policy will be applied.
            - The Provider Virtual Network must already be added to a Fabric Site before applying the policy.
            - Updating this field is allowed, but once an extranet policy is applied to a site, it cannot be removed.
            - Fabric Site(s) connected to the same SD-Access Transit must have consistent Extranet Policies.
            - Selecting a Fabric Site connected to an SD-Access Transit will automatically select all other Sites connected to that Transit.
            - Example - ["Global/USA/San Jose/Building23", "Global/India/Bangalore/Building18"]
        type: list
        elements: str


requirements:
- dnacentersdk == 2.7.0
- python >= 3.9
notes:
  - SDK Methods used are
    sites.Sites.get_site
    sda.SDA.get_fabric_sites
    sda.SDA.get_extranet_policies
    sda.SDA.add_extranet_policy
    sda.SDA.update_extranet_policy
    sda.SDA.delete_extranet_policy_by_id
    task.Task.get_task_by_id

  - Paths used are
    get /dna/intent/api/v1/site
    get /dna/intent/api/v1/sda/fabricSites
    get /dna/intent/api/v1/sda/extranetPolicies
    post /dna/intent/api/v1/sda/extranetPolicies
    put /dna/intent/api/v1/sda/extranetPolicies
    delete dna/intent/api/v1/sda/extranetPolicies/${id}
    get /dna/intent/api/v1/task/{taskId}

"""

EXAMPLES = r"""
- name: Create Extranet Policy
  cisco.dnac.sda_extranet_policies_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log_level: "{{dnac_log_level}}"
    dnac_log: true
    state: merged
    config:
      - extranet_policy_name: "test_extranet_policy_1"
        provider_virtual_network: "VN_1"
        subscriber_virtual_networks: ["VN_2", "VN_3"]

- name: Create Extranet Policy with Fabric Site(s) specified
  cisco.dnac.sda_extranet_policies_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log_level: "{{dnac_log_level}}"
    dnac_log: true
    state: merged
    config:
      - extranet_policy_name: "test_extranet_policy_1"
        provider_virtual_network: "VN_1"
        subscriber_virtual_networks: ["VN_2", "VN_3"]
        fabric_sites: ["Global/Test_Extranet_Polcies/USA", "Global/Test_Extranet_Polcies/India"]

- name: Update existing Extranet Policy
  cisco.dnac.sda_extranet_policies_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log_level: "{{dnac_log_level}}"
    dnac_log: true
    state: merged
    config:
      - extranet_policy_name: "test_extranet_policy_1"
        provider_virtual_network: "VN_1"
        subscriber_virtual_networks: ["VN_2", "VN_4"]

- name: Update existing Extranet Policy with Fabric Site(s) specified
  cisco.dnac.sda_extranet_policies_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log_level: "{{dnac_log_level}}"
    dnac_log: true
    state: merged
    config:
      - extranet_policy_name: "test_extranet_policy_1"
        fabric_sites: ["Global/Test_Extranet_Polcies/USA", "Global/Test_Extranet_Polcies/India"]
        provider_virtual_network: "VN_1"
        subscriber_virtual_networks: ["VN_2", "VN_4"]

- name: Delete Extranet Policy
  cisco.dnac.sda_extranet_policies_workflow_manager:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    dnac_log_level: "{{dnac_log_level}}"
    dnac_log: true
    state: deleted
    config:
      - extranet_policy_name: "test_extranet_policy_1"
"""

RETURN = r"""
#Case_1: Response when task is successful
sample_response_2:
  description: A dictionary with the response returned by the Cisco Catalyst Center Python SDK
  returned: always
  type: dict
  sample: >
    {
      "status": "string",
      "changed": bool,
      "msg": "string"
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }

#Case_3: Response when Error Occurs
sample_response_3:
  description: A dictionary with the response returned by the Cisco Catalyst Center Python SDK
  returned: always
  type: dict
  sample: >
    {
      "changed": bool,
      "msg": "string"
    }
"""

import time
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.dnac.plugins.module_utils.dnac import (
    DnacBase,
    validate_list_of_dicts
)


class SDAExtranetPolicies(DnacBase):
    """
    A class for managing Extranet Policies within the Cisco DNA Center using the SDA API.
    """
    def __init__(self, module):
        """
        Initialize an instance of the class.
        Parameters:
          - module: The module associated with the class instance.
        Returns:
          The method does not return a value.
        """
        super().__init__(module)

    def validate_input(self):
        """
        Validates the input configuration parameters for the playbook.
        Returns:
            object: An instance of the class with updated attributes:
                - self.msg: A message describing the validation result.
                - self.status: The status of the validation (either "success" or "failed").
                - self.validated_config: If successful, a validated version of the "config" parameter.

        Description:
            This method validates the fields provided in the playbook against a predefined specification.
            It checks if the required fields are present and if their data types match the expected types.
            If any parameter is found to be invalid, it logs an error message and sets the validation status to "failed".
            If the validation is successful, it logs a success message and returns an instance of the class
            with the validated configuration.
        """
        # Check if configuration is available
        if not self.config:
            self.status = "success"
            self.msg = "Configuration is not available in the playbook for validation"
            self.log(self.msg, "ERROR")
            return self

        # Expected schema for configuration parameters
        temp_spec = {
            "extranet_policy_name": {"type": "str", "required": True},
            "fabric_sites": {"type": "list", "elements": "str", "required": False},
            "provider_virtual_network": {"type": "str", "required": False},
            "subscriber_virtual_networks": {"type": "list", "elements": "str", "required": False},
        }

        # Validate params
        valid_temp, invalid_params = validate_list_of_dicts(
            self.config, temp_spec
        )

        if invalid_params:
            self.msg = "Invalid parameters in playbook: {0}".format(invalid_params)
            self.set_operation_result("failed", False, self.msg, "ERROR")
            return self

        # Set the validated configuration and update the result with success status
        self.validated_config = valid_temp
        self.msg = "Successfully validated playbook configuration parameters using 'validated_input': {0}".format(str(valid_temp))
        self.set_operation_result("success", False, self.msg, "INFO")
        return self

    def get_fabric_ids_list(self, site_details):
        """
        Extracts a list of fabric IDs from the provided site details.
        Parameters:
            - site_details (dict): A dictionary containing site information. Each key-value pair
                                   represents a site, where the value is another dictionary that
                                   includes a 'fabric_id'.
        Returns:
            list: A list of fabric IDs extracted from the site details.
        Description:
            This method iterates over the values in the provided site_details dictionary, extracts
            the 'fabric_id' from each value, and appends it to a list. The resulting list of fabric IDs
            is then returned.
        """
        # Initialize an empty list to store fabric IDs
        fabric_ids_list = []

        # Iterate over each site's information in the site details
        for site_info in site_details.values():
            fabric_ids_list.append(site_info["fabric_id"])
        return fabric_ids_list

    def validate_merged_parameters(self, config):
        """
        Validate that the required parameters are present in the configuration for performing
        Add or Update Extranet Policy operations.
        Parameters:
            - config (dict): A dictionary containing the configuration parameters to be validated.
        Returns:
            None: This function does not return a value. It logs messages and raises exceptions
                  if required parameters are missing.
        Description:
            This method checks the provided configuration for the presence of the required parameters:
            'provider_virtual_network' and 'subscriber_virtual_networks'. If any of these parameters
            are missing, it logs an error message and raises an exception to halt execution. If all
            required parameters are present, it logs a success message indicating successful validation.
        """
        # Check for provider_virtual_network
        provider_virtual_network = config.get("provider_virtual_network")
        if provider_virtual_network is None:
            msg = ("Missing required parameter: 'provider_virtual_network'. "
                   "(extranet_policy_name, provider_virtual_network, and subscriber_virtual_networks) - "
                   "are the required parameters for performing Add or Update Extranet Policy operations.")
            self.log(msg, "ERROR")
            self.module.fail_json(msg)

        # Check for subscriber_virtual_networks
        subscriber_virtual_networks = config.get("subscriber_virtual_networks")
        if subscriber_virtual_networks is None:
            msg = (
                "Missing required parameter: 'subscriber_virtual_networks'. "
                "(extranet_policy_name, provider_virtual_network, and subscriber_virtual_networks) - "
                "are the required parameters for performing Add or Update Extranet Policy operations."
            )
            self.log(msg, "ERROR")
            self.module.fail_json(msg)

        self.log(
            "Successfully validated that the required parameters — (extranet_policy_name, "
            "provider_virtual_network, and subscriber_virtual_networks) are provided",
            "INFO"
        )

    def get_add_extranet_policy_params(self, config, site_details=None):
        """
        Generate parameters required for adding an Extranet Policy based on the provided configuration and site details.
        Parameters:
            - config (dict): A dictionary containing the configuration parameters.
            - site_details (dict, optional): A dictionary containing site details. Default is None.
        Returns:
            dict: A dictionary containing the parameters for adding an Extranet Policy.
        Description:
            This method constructs a dictionary of parameters required for adding an Extranet Policy.
            It includes the 'extranetPolicyName', 'providerVirtualNetworkName', and 'subscriberVirtualNetworkNames'
            from the configuration. If 'fabric_sites' are provided in the configuration and site details are available,
            it also includes the 'fabricIds' obtained from the site details.
        """
        # Initialize the parameters dictionary with basic required parameters
        add_extranet_policy_params = {
            "extranetPolicyName": config.get("extranet_policy_name"),
            "providerVirtualNetworkName": config.get("provider_virtual_network"),
            "subscriberVirtualNetworkNames": config.get("subscriber_virtual_networks")
        }

        # Check if 'fabric_sites' are provided and site details are available
        if config.get("fabric_sites") and site_details:
            add_extranet_policy_params["fabricIds"] = self.get_fabric_ids_list(site_details)
        else:
            add_extranet_policy_params["fabricIds"] = []

        return add_extranet_policy_params

    def get_update_extranet_policy_params(self, config, extranet_policy_id, site_details=None):
        """
        Generate parameters required for updating an Extranet Policy based on the provided configuration,
        policy ID, and site details.
        Parameters:
            config (dict): A dictionary containing the configuration parameters.
            extranet_policy_id (str): The ID of the Extranet Policy to be updated.
            site_details (dict, optional): A dictionary containing site details. Default is None.
        Returns:
            dict: A dictionary containing the parameters for updating an Extranet Policy.
        Description:
            This method constructs a dictionary of parameters required for updating an Extranet Policy.
            It includes the 'id' of the policy, 'extranetPolicyName', 'providerVirtualNetworkName', and
            'subscriberVirtualNetworkNames' from the configuration. If 'fabric_sites' are provided in the
            configuration and site details are available, it also includes the 'fabricIds' obtained from the
            site details.
        """
        # Initialize the parameters dictionary with basic required parameters
        update_extranet_policy_params = {
            "id": extranet_policy_id,
            "extranetPolicyName": config.get("extranet_policy_name"),
            "providerVirtualNetworkName": config.get("provider_virtual_network"),
            "subscriberVirtualNetworkNames": config.get("subscriber_virtual_networks")
        }

        # Check if 'fabric_sites' are provided and site details are available
        if config.get("fabric_sites") and site_details:
            update_extranet_policy_params["fabricIds"] = self.get_fabric_ids_list(site_details)
        else:
            update_extranet_policy_params["fabricIds"] = []

        return update_extranet_policy_params

    def get_delete_extranet_policy_params(self, extranet_policy_id):
        """
        Generate parameters required for deleting an Extranet Policy based on the provided policy ID.
        Parameters:
            extranet_policy_id (str): The unique identifier of the Extranet Policy to be deleted.
        Returns:
            dict: A dictionary containing the parameters for deleting an Extranet Policy.
        Description:
            This method constructs a dictionary of parameters required for deleting an Extranet Policy.
            It includes the 'id' of the policy, which is necessary for identifying the specific policy
            to be deleted.
        """
        # Create a dictionary with the extranet policy ID
        delete_extranet_policy_params = {
            "id": extranet_policy_id
        }

        return delete_extranet_policy_params

    def get_site_details(self, fabric_sites):
        """
        Retrieve details for each site in the provided fabric sites list.
        Parameters:
            - fabric_sites (list): A list of site names to be validated and detailed.
        Returns:
            dict: A dictionary containing the details for each site, including existence and site ID.
        Description:
            This method takes a list of fabric sites and checks if each site exists using the validate_site_exists method.
            It constructs a dictionary where each key is a site name and the value is another dictionary containing
            'site_exists' (a boolean indicating if the site exists) and 'site_id' (the unique identifier of the site).
        """
        # Initialize an empty dictionary to store site details
        site_details = {}

        # Iterate over each site in the provided fabric sites list
        for site in fabric_sites:
            self.log("Starting to retrieve site details for the provided fabric site: {0}".format(site), "INFO")
            # Validate if the site exists and retrieve its ID
            site_exists, site_id = self.get_site_id(site)
            self.log("Site details for '{0}': exists={1}, id={2}".format(site, site_exists, site_id), "INFO")
            site_details[site] = {
                "site_exists": site_exists,
                "site_id": site_id,
            }

        return site_details

    def get_fabric_sites(self, site_name, site_id):
        """
        Retrieve the fabric ID for a given site using the SDA 'get_fabric_sites' API call.
        Parameters:
            - site_name (str): The name of the site.
            - site_id (str): The unique identifier of the site.
        Returns:
            str: The fabric ID if found, otherwise None.
        Description:
            This method calls the SDA 'get_fabric_sites' API to retrieve the fabric ID for a specified site. It logs the response,
            processes the response to extract the fabric ID, and handles any exceptions that occur during the API call.
        """
        try:
            # Call the SDA 'get_fabric_sites' API with the provided site ID
            response = self.dnac._exec(
                family="sda",
                function="get_fabric_sites",
                op_modifies=False,
                params={"siteId": site_id},
            )
            self.log("Response received post SDA - 'get_fabric_sites' API call: {0}".format(str(response)), "DEBUG")

            response = response["response"]

            if not response:
                self.log("No response received from the SDA - 'get_fabric_sites' API call.", "WARNING")
                return None

            # Process the response if available
            fabric_id = response[0]["id"]
            self.log("Successfully retrieved fabric ID: '{0}' for Site: '{1}'".format(fabric_id, site_name), "INFO")

            return fabric_id

        except Exception as e:
            # Log an error message and fail if an exception occurs
            self.msg = (
                "An error occurred while retrieving fabric Site 'Id' for Site '{0}' using SDA - "
                "'get_fabric_sites' API call: {1}".format(site_name, str(e))
            )
            self.fail_and_exit(self.msg)

    def get_fabric_sites_ids(self, site_details):
        """
        Retrieve and update fabric IDs for a list of sites.
        Parameters:
            - site_details (dict): A dictionary where each key is a site name and the value is another dictionary
              containing site information, including "site_id".
        Returns:
            dict: The updated dictionary with fabric IDs added to each site's information.
        Description:
            This method iterates through the provided `site_details` dictionary, retrieves the fabric ID for each site
            by calling the `get_fabric_sites` method, and logs the retrieved fabric IDs along with site details.
            It updates the `site_details` dictionary to include the fabric ID for each site and logs the updated
            information.
        """
        for site_name, site_info in site_details.items():
            site_id = site_info["site_id"]
            # Get the fabric ID using the site name and site ID
            fabric_id = self.get_fabric_sites(site_name, site_id)
            if fabric_id is not None:
                self.log("Fabric ID: {0} collected for the fabric site: {1} with siteId: {2}".format(fabric_id, site_name, site_id), "INFO")
                site_info["fabric_id"] = fabric_id
            else:
                self.msg = "Failed to retrieve Fabric ID for site: {0} with siteId: {1}".format(site_name, site_id)
                self.fail_and_exit(self.msg)
        self.log("Updated 'site_details' with the fabric_ids of each site.  {0}".format(site_details))
        return site_details

    def get_extranet_policies(self, extranet_policy_name):
        """
        Retrieve extranet policies for a given policy name using the SDA 'get_extranet_policies' API call.
        Parameters:
            - extranet_policy_name (str): The name of the extranet policy to retrieve.
        Returns:
            dict or None: The response dictionary containing policy details if found, otherwise None.
        Description:
            This method calls the SDA 'get_extranet_policies' API to retrieve details for the specified extranet
            policy name. It logs the response received from the API call and processes it. If the API call is successful
            and returns data, the first item in the response is returned. If no data is received or an exception occurs,
            appropriate warnings or error messages are logged.
        """
        try:
            # Execute the API call to get extranet policie
            response = self.dnac._exec(
                family="sda",
                function="get_extranet_policies",
                op_modifies=False,
                params={"extranetPolicyName": extranet_policy_name},
            )
            self.log("Response received post SDA - 'get_extranet_policies' API call: {0}".format(str(response)), "DEBUG")

            # Process the response if available
            response = response["response"]
            if not response:
                self.log("No response received from the SDA - 'get_extranet_policies' API call.", "WARNING")
                return None
            return response[0]

        except Exception as e:
            # Log an error message and fail if an exception occurs
            self.msg = (
                "An error occurred while retrieving Extranet Policy Details: '{0}' using SDA - "
                "'get_extranet_policies' API call: {1}".format(extranet_policy_name, str(e))
            )
            self.fail_and_exit(self.msg)

    def validate_extranet_policy_exists(self, extranet_policy_name):
        """
        Check if an extranet policy exists and retrieve its details.
        Parameters:
            - config (dict): A dictionary containing configuration details, including the key "extranet_policy_name".
        Returns:
            tuple: A tuple containing:
                - bool: `True` if the extranet policy exists, otherwise `False`.
                - str or None: The ID of the extranet policy if it exists, otherwise `None`.
                - dict or None: The details of the extranet policy if it exists, otherwise `None`.
        Description:
            This method verifies the existence of an extranet policy based on the name provided in the `config` dictionary.
            It calls the `get_extranet_policies` method to retrieve policy details. If the policy is found, it sets
            `extranet_policy_exists` to `True` and extracts the policy ID and details. The method returns a tuple containing
            the existence status, policy ID, and policy details.
        """
        # Initialize variables to default values
        extranet_policy_exists = False
        extranet_policy_id = None

        self.log("Validating existence of Extranet Policy: {0}".format(extranet_policy_name), "INFO")

        extranet_policy_details = self.get_extranet_policies(extranet_policy_name)

        # Check if the policy details were retrieved successfully
        if extranet_policy_details:
            extranet_policy_exists = True
            extranet_policy_id = extranet_policy_details["id"]
            self.log("Extranet Policy: '{0}' exists with ID: {1}".format(extranet_policy_name, extranet_policy_id), "INFO")
        else:
            self.log("Extranet Policy: '{0}' does not exist.".format(extranet_policy_name), "WARNING")

        return (extranet_policy_exists, extranet_policy_id, extranet_policy_details)

    def compare_extranet_policies(self, extranet_policy_details, update_extranet_policy_params):
        """
        Compare the details of two extranet policies to check if they are equivalent.
        Parameters:
            - extranet_policy_details (dict): A dictionary containing the current details of the extranet policy.
            - update_extranet_policy_params (dict): A dictionary containing the updated policy parameters to compare against.
        Returns:
            bool: `True` if all values for the keys match between the two dictionaries, `False` otherwise.
        Description:
            This method compares the details of two extranet policies by iterating over each key in the `extranet_policy_details`
            dictionary and checking if the corresponding values in the `update_extranet_policy_params` dictionary match.
            Lists are compared regardless of order, while other values are compared directly. The method returns `True` if
            all values are equivalent, and `False` if any values differ.
        """
        # Iterate over each key in the extranet policy details and compare the details
        for key in extranet_policy_details:
            current_value = extranet_policy_details.get(key)
            requested_value = update_extranet_policy_params.get(key)

            self.log("Comparing key: {0}, existing_value: {1}, requested_value: {2}".format(key, current_value, requested_value), "INFO")

            if key == "fabricIds":
                if current_value and not requested_value:
                    self.log("Skipping comparison for key: 'fabricIds' as the requested value is empty.", "DEBUG")
                    continue

            if isinstance(current_value, list) and isinstance(requested_value, list):
                # Compare lists regardless of order
                if sorted(current_value) != sorted(requested_value):
                    self.log("Mismatch found for key: {0}, existing list: {1}, requested list: {2}".format(key, current_value, requested_value), "INFO")
                    return False
            else:
                # Compare values directly
                if current_value != requested_value:
                    self.log("Mismatch found for key: {0}, existing list: {1}, requested list: {2}".format(key, current_value, requested_value), "INFO")
                    return False

        self.log("All keys and values match between the existing and requested policies.", "INFO")

        return True

    def add_extranet_policy(self, add_extranet_policy_params):
        """
        Adds an extranet policy by making a POST API call with the provided parameters.
        Args:
            add_extranet_policy_params (dict): Parameters for adding the extranet policy.
        Returns:
            str: Task ID for the add extranet policy operation.
        """
        # Wrap the parameters in a payload dictionary
        add_extranet_policy_params = {"payload": [add_extranet_policy_params]}

        # Make the API call to add the extranet policy and return the task ID
        return self.get_taskid_post_api_call("sda", "add_extranet_policy", add_extranet_policy_params)

    def get_add_extranet_policy_status(self, task_id):
        """
        Retrieves the status of the add extranet policy task using the provided task ID.
        Args:
            task_id (str): The task ID to check the status for.
        Returns:
            dict: The status of the add extranet policy task.
        """
        task_name = "Add Extranet Policy Task"
        msg = {}

        # Get the name of the extranet policy from the input parameters
        extranet_policy_name = self.want.get("add_extranet_policy_params").get("extranetPolicyName")
        msg["{0} Succeeded for the Extranet Policy".format(task_name)] = extranet_policy_name

        # Retrieve and return the task status using the provided task ID
        return self.get_task_status_from_tasks_by_id(task_id, task_name, msg)

    def update_extranet_policy(self, update_extranet_policy_params):
        """
        Updates an existing extranet policy by making a POST API call with the provided parameters.
        Args:
            update_extranet_policy_params (dict): Parameters for updating the extranet policy.
        Returns:
            str: Task ID for the update extranet policy operation.
        """
        # Wrap the parameters in a payload dictionary
        update_extranet_policy_params = {"payload": [update_extranet_policy_params]}

        # Make the API call to update the extranet policy and return the task ID
        return self.get_taskid_post_api_call("sda", "update_extranet_policy", update_extranet_policy_params)

    def get_update_extranet_policy_status(self, task_id):
        """
        Retrieves the status of the update extranet policy task using the provided task ID.
        Args:
            task_id (str): The task ID to check the status for.
        Returns:
            dict: The status of the update extranet policy task.
        """
        task_name = "Update Extranet Policy Task"
        msg = {}

        # Get the name of the extranet policy from the input parameters
        extranet_policy_name = self.want.get("update_extranet_policy_params").get("extranetPolicyName")
        msg["{0} Succeeded for following Extranet Policy".format(task_name)] = extranet_policy_name

        # Retrieve and return the task status using the provided task ID
        return self.get_task_status_from_tasks_by_id(task_id, task_name, msg)

    def delete_extranet_policy(self, delete_extranet_policy_params):
        """
        Deletes an existing extranet policy by making a POST API call with the provided parameters.
        Args:
            delete_extranet_policy_params (dict): Parameters for deleting the extranet policy.
        Returns:
            str: Task ID for the delete extranet policy operation.
        """
        # Make the API call to delete the extranet policy and return the task ID
        return self.get_taskid_post_api_call("sda", "delete_extranet_policy_by_id", delete_extranet_policy_params)

    def get_delete_extranet_policy_status(self, task_id):
        """
        Retrieves the status of the delete extranet policy task using the provided task ID.
        Args:
            task_id (str): The task ID to check the status for.
        Returns:
            dict: The status of the delete extranet policy task.
        """
        task_name = "Delete Extranet Policy Task"
        msg = {}

        # Get the name of the extranet policy from the input parameters
        extranet_policy_name = self.want.get("extranet_policy_name")
        msg["{0} Succeeded for following Extranet Policy".format(task_name)] = extranet_policy_name

        # Retrieve and return the task status using the provided task ID
        return self.get_task_status_from_tasks_by_id(task_id, task_name, msg)

    def get_have(self, config):
        """
        Retrieve the current state of the extranet policy based on the provided configuration.
        Parameters:
            - config (dict): Configuration dictionary containing site details.
        Returns:
            self: The instance of the class, allowing for method chaining.
        Description:
            This method checks if the extranet policy specified in the `config` exists. It uses the
            `validate_extranet_policy_exists` method to determine if the policy exists and to retrieve its details.
            The method logs the current state of the extranet policy and updates the instance attribute `have` with
            information about the existence, ID, and details of the extranet policy. It returns the instance for
            method chaining.
        """
        have = {}

        extranet_policy_name = config.get("extranet_policy_name")
        # check if given extranet policy exits, if exists store current extranet policy info
        (extranet_policy_exists, extranet_policy_id, extranet_policy_details) = self.validate_extranet_policy_exists(extranet_policy_name)

        self.log("Current Extranet Policy details (have): {0}".format(str(extranet_policy_details)), "DEBUG")

        have["extranet_policy_exists"] = extranet_policy_exists
        have["extranet_policy_id"] = extranet_policy_id
        have["current_extranet_policy"] = extranet_policy_details

        self.have = have
        self.log("Current Extranet Policy State (have): {0}".format(str(self.have)), "INFO")

        return self

    def get_want(self, config, state):
        """
        Generate the desired state parameters for API calls based on the provided configuration and state.
        Parameters:
            - config (dict): Configuration dictionary containing site and policy details.
            - state (str): Desired state, which can be 'merged' or 'delete'.
        Returns:
            self: The instance of the class, allowing for method chaining.
        Description:
            This method determines the parameters required for API calls based on the desired state and configuration.
            It checks if the extranet policy exists and sets the appropriate parameters for creating, updating, or deleting
            the policy. For the 'merged' state, it prepares parameters for updating the policy if it exists or creating
            it if it does not. For the 'delete' state, it prepares parameters for deleting the policy if it exists. The
            method logs the created parameters and updates the instance attribute `want` with these parameters. It returns
            the instance for method chaining.
        """
        # Initialize want
        want = {}
        site_details = {}

        self.log("Creating Parameters for API Calls with state: {0}".format(state))

        # Identify if policy already exists or needs to be created
        extranet_policy_name = config.get("extranet_policy_name")
        extranet_policy_exists = self.have.get("extranet_policy_exists")
        extranet_policy_id = self.have.get("extranet_policy_id")
        extranet_policy_details = self.have.get("current_extranet_policy")

        if state == "merged":
            self.validate_merged_parameters(config)
            fabric_sites = config.get("fabric_sites")
            if fabric_sites:
                self.log("Attempting to get the 'site ID' for the provided fabric sites: {0}".format(fabric_sites), "DEBUG")
                site_details = self.get_site_details(fabric_sites)
                self.log("Attempting to get the 'fabric ID' for the provided fabric sites: {0}".format(fabric_sites), "DEBUG")
                site_details = self.get_fabric_sites_ids(site_details)

            if extranet_policy_exists:
                self.log(
                    "Extranet Policy - '{0}' exists in the Cisco Catalyst Center, "
                    "therefore setting 'update_extranet_policy_params'.".format(extranet_policy_name),
                    "DEBUG"
                )
                want = dict(update_extranet_policy_params=self.get_update_extranet_policy_params(config, extranet_policy_id, site_details))
                if self.compare_extranet_policies(extranet_policy_details, want["update_extranet_policy_params"]):
                    self.msg = (
                        "Extranet Policy '{0}' is identical to the update requested. No update operation needed.".format(extranet_policy_name)
                    )
                    self.set_operation_result("ok", False, self.msg, "INFO")
                    self.check_return_status()
                    return self
            else:
                self.log(
                    "Extranet Policy - '{0}' does not exist in the Cisco Catalyst Center, "
                    "therefore setting 'add_extranet_policy_params'.".format(extranet_policy_name),
                    "DEBUG"
                )
                want = dict(add_extranet_policy_params=self.get_add_extranet_policy_params(config, site_details))

        elif state == "deleted":
            if extranet_policy_exists:
                self.log(
                    "State is delete and Extranet Policy - '{0}' exists in the Cisco Catalyst Center, "
                    "therefore setting 'delete_extranet_policy_params'.".format(extranet_policy_name),
                    "DEBUG"
                )
                want = dict(extranet_policy_name=extranet_policy_name,
                            delete_extranet_policy_params=self.get_delete_extranet_policy_params(extranet_policy_id))
            else:
                self.msg = (
                    "Extranet Policy - '{0}' does not exist in the Cisco Catalyst Center and "
                    "hence delete operation not required.".format(extranet_policy_name)
                )
                self.set_operation_result("ok", False, self.msg, "INFO")
                self.check_return_status()
                return self

        self.want = want
        self.log("Desired State (want): {0}".format(str(self.want)), "INFO")
        self.msg = "Successfully collected all parameters from the playbook for creating/updating/deleting the extranet policy."
        self.status = "success"
        return self

    def get_diff_merged(self):
        """
        Executes actions based on the desired state parameters and checks their status.
        Parameters:
            - None
        Returns:
            self: The instance of the class, allowing for method chaining.
        Description:
            This method iterates through a map of action parameters to their corresponding functions for execution and status
            checking. For each action parameter present in the desired state (`want`), the associated action function is called
            to perform the action, and the corresponding status function is used to check the result. It ensures that all actions
            specified in the desired state are executed and their statuses are verified. The method returns the instance for method
            chaining.
        """
        self.log("Starting 'get_diff_merged' operation.", "INFO")
        action_map = {
            "add_extranet_policy_params": (self.add_extranet_policy, self.get_add_extranet_policy_status),
            "update_extranet_policy_params": (self.update_extranet_policy, self.get_update_extranet_policy_status),
        }

        for action_param, (action_func, status_func) in action_map.items():
            # Execute the action and check its status
            req_action_param = self.want.get(action_param)
            if req_action_param:
                self.log("Executing action for parameter: {0}".format(req_action_param), "INFO")
                result_task_id = action_func(req_action_param)
                status_func(result_task_id).check_return_status()

        self.log("Completed 'get_diff_merged' operation.", "INFO")
        return self

    def get_diff_deleted(self):
        """
        Executes deletion actions based on the desired state parameters and checks their status.
        Parameters:
            - None
        Returns:
            self: The instance of the class, allowing for method chaining.
        Description:
            This method iterates through a map of deletion action parameters to their corresponding functions for execution and
            status checking. For each deletion action parameter present in the desired state (`want`), the associated action
            function is called to perform the deletion, and the corresponding status function is used to check the result.
            It ensures that all deletion actions specified in the desired state are executed and their statuses are verified.
            The method returns the instance for method chaining.
        """
        self.log("Starting 'get_diff_deleted' operation.", "INFO")
        action_map = {
            "delete_extranet_policy_params": (self.delete_extranet_policy, self.get_delete_extranet_policy_status)

        }
        for action_param, (action_func, status_func) in action_map.items():
            # Execute the action and check its status
            if self.want.get(action_param):
                result_task_id = action_func(self.want.get(action_param))
                status_func(result_task_id).check_return_status()

        self.log("Completed 'get_diff_deleted' operation.", "INFO")
        return self

    def verify_diff_merged(self, config):
        """
        Verifies the results of the merged state operations by comparing the state before and after the operations.
        Parameters:
            - config (dict): Configuration dictionary containing site and policy details.
        Returns:
            self: The instance of the class, allowing for method chaining.
        Description:
            This method performs verification of operations related to the 'merged' state. It first retrieves the state before
            performing any operations and then compares it with the state after the operations. For add and update operations,
            it logs the states before and after the operations and verifies the success based on the presence or absence of
            the extranet policy and whether any changes were detected. It ensures that the operations have been performed as
            expected and logs appropriate messages based on the results.
        """
        self.log("Starting 'verify_diff_merged' operation.", "INFO")

        pre_operation_state = self.have.copy()
        desired_state = self.want
        self.get_have(config)
        post_operation_state = self.have.copy()
        extranet_policy_name = config.get("extranet_policy_name")

        add_extranet_policy_params = desired_state.get("add_extranet_policy_params")
        if add_extranet_policy_params:
            self.log("State before performing ADD Extranet Policy operation: {0}".format(str(pre_operation_state)), "INFO")
            self.log("Desired State: {0}".format(str(add_extranet_policy_params)), "INFO")
            self.log("State after performing ADD Extranet Policy operation: {0}".format(str(post_operation_state)), "INFO")

            if post_operation_state["extranet_policy_exists"]:
                self.log("Verified the success of ADD Extranet Policy - '{0}' operation.".format(extranet_policy_name), "INFO")
            else:
                self.log(
                    "The ADD Extranet Policy - '{0}' operation may not have been successful "
                    "since the Extranet Policy does not exist in the Cisco Catalyst Center.".format(extranet_policy_name),
                    "WARNING"
                )
                self.log("Completed verification of ADD Extranet Policy operation.", "INFO")

        update_extranet_policy_params = desired_state.get("update_extranet_policy_params")
        if update_extranet_policy_params:
            self.log("State before performing UPDATE Extranet Policy operation: {0}".format(str(pre_operation_state)), "INFO")
            self.log("Desired State: {0}".format(str(update_extranet_policy_params)), "INFO")
            self.log("State after performing UPDATE Extranet Policy operation - '{0}'".format(str(post_operation_state)), "INFO")

            if not self.compare_extranet_policies(pre_operation_state["current_extranet_policy"], post_operation_state["current_extranet_policy"]):
                self.log("Verified the success of UPDATE Extranet Policy - '{0}' operation.".format(extranet_policy_name), "INFO")
            else:
                self.log(
                    "The UPDATE Extranet Policy - '{0}' operation may not have been performed or "
                    "may not have been successful because no change was detected in the Extranet Policy "
                    "in the Cisco Catalyst Center".format(extranet_policy_name),
                    "WARNING"
                )
                self.log("Completed verification of UPDATE Extranet Policy operation.", "INFO")

        self.log("Completed 'verify_diff_merged' operation.", "INFO")
        return self

    def verify_diff_deleted(self, config):
        """
        Verifies the results of the delete state operation by comparing the state before and after the delete operation.
        Parameters:
            - config (dict): Configuration dictionary containing site and policy details.
        Returns:
            self: The instance of the class, allowing for method chaining.
        Description:
            This method performs verification of the delete operation by comparing the state before and after the operation.
            It introduces a delay to allow the deletion to process and then retrieves the state. It checks if the extranet policy
            no longer exists and logs the result of the delete operation. It ensures that the delete operation was successful
            by verifying the absence of the extranet policy and logs appropriate messages based on the outcome.
        """
        self.log("Starting 'verify_diff_deleted' operation.", "INFO")

        pre_operation_state = self.have.copy()
        desired_state = self.want
        time.sleep(10)
        self.get_have(config)
        post_operation_state = self.have.copy()
        extranet_policy_name = config.get("extranet_policy_name")

        self.log("State before performing DELETE Extranet Policy operation: {0}".format(str(pre_operation_state)), "INFO")
        self.log("Desired State: {0}".format(str(desired_state)), "INFO")
        self.log("State after performing DELETE Extranet Policy operation: {0}".format(str(post_operation_state)), "INFO")

        if not post_operation_state["extranet_policy_exists"]:
            self.log("Verified the success of DELETE Extranet Policy - '{0}' operation".format(extranet_policy_name), "INFO")
        else:
            self.log(
                "The DELETE Extranet Policy - '{0}' operation may not have been successful since "
                "the policy still exists in the Cisco Catalyst Center.".format(extranet_policy_name),
                "WARNING"
            )

        self.log("Completed 'verify_diff_deleted' operation.", "INFO")
        return self


def main():
    """ main entry point for module execution
    """
    # Define the specification for the module"s arguments
    element_spec = {
        "dnac_host": {"required": True, "type": "str"},
        "dnac_port": {"type": "str", "default": "443"},
        "dnac_username": {"type": "str", "default": "admin", "aliases": ["user"]},
        "dnac_password": {"type": "str", "no_log": True},
        "dnac_verify": {"type": "bool", "default": "True"},
        "dnac_version": {"type": "str", "default": "2.2.3.3"},
        "dnac_debug": {"type": "bool", "default": False},
        "dnac_log_level": {"type": "str", "default": "WARNING"},
        "dnac_log_file_path": {"type": "str", "default": "dnac.log"},
        "dnac_log_append": {"type": "bool", "default": True},
        "dnac_log": {"type": "bool", "default": False},
        "validate_response_schema": {"type": "bool", "default": True},
        "config_verify": {"type": "bool", "default": False},
        "dnac_api_task_timeout": {"type": "int", "default": 1200},
        "dnac_task_poll_interval": {"type": "int", "default": 2},
        "config": {"required": True, "type": "list", "elements": "dict"},
        "state": {"default": "merged", "choices": ["merged", "deleted"]}
    }

    # Initialize the Ansible module with the provided argument specifications
    module = AnsibleModule(argument_spec=element_spec,
                           supports_check_mode=False)

    # Initialize the NetworkCompliance object with the module
    ccc_sda_extranet_policies = SDAExtranetPolicies(module)

    # Get the state parameter from the provided parameters
    state = ccc_sda_extranet_policies.params.get("state")

    # Check if the state is valid
    if state not in ccc_sda_extranet_policies.supported_states:
        ccc_sda_extranet_policies.status = "invalid"
        ccc_sda_extranet_policies.msg = "State {0} is invalid".format(state)
        ccc_sda_extranet_policies.check_return_status()

    # Validate the input parameters and check the return status
    ccc_sda_extranet_policies.validate_input().check_return_status()

    # Get the config_verify parameter from the provided parameters
    config_verify = ccc_sda_extranet_policies.params.get("config_verify")

    # Iterate over the validated configuration parameters
    for config in ccc_sda_extranet_policies.validated_config:
        ccc_sda_extranet_policies.reset_values()
        ccc_sda_extranet_policies.get_have(config).check_return_status()
        ccc_sda_extranet_policies.get_want(config, state).check_return_status()
        ccc_sda_extranet_policies.get_diff_state_apply[state]().check_return_status()

        if config_verify:
            ccc_sda_extranet_policies.verify_diff_state_apply[state](config).check_return_status()

    module.exit_json(**ccc_sda_extranet_policies.result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2024 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_storageaccountmanagementpolicy_info

version_added: "2.4.0"

short_description: Get the data policy rules associated with the specified storage account

description:
    - Get the data policy rules associated with the specified storage account.

options:
    resource_group:
        description:
            - Name of the resource group to use.
        required: true
        type: str
        aliases:
            - resource_group_name
    storage_account_name:
        description:
            - Name of the storage account to update or create.
        type: str
        required: true

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Get the data policy rules associated with the specified storage account
  azure_rm_storageaccountmanagementpolicy_info:
    resource_group: myResourceGroup
    storage_account_name: testaccount
'''


RETURN = '''
state:
    description:
        - The data policy rules associated with the specified storage account.
    returned: always
    type: complex
    contains:
        id:
            description:
                - The data policy's ID.
            returned: always
            type: str
            sample: "/subscriptions/xxx-xxx/resourceGroups/testRG/providers/Microsoft.Storage/storageAccounts/sttest/managementPolicies/default"
        resource_group:
            description:
                - The resource group name.
            returned: always
            type: str
            sample: testRG
        storage_account_name:
            description:
                - The storage account name.
            returned: always
            type: str
            sample: teststname
        type:
            description:
                - The type of the resource.
            returned: always
            type: str
            sample: "Microsoft.Storage/storageAccounts/managementPolicies"
        last_modified_time:
            description:
                - Returns the date and time the ManagementPolicies was last modified.
            returned: always
            type: str
            sample: "2024-04-12T11:40:10.376465+00:00"
        name:
            description:
                - The name of the resource.
            returned: always
            type: str
            sample: DefaultManagementPolicy
        policy:
            description:
                - The Storage Account ManagementPolicy.
            returned: always
            type: complex
            contains:
                rules:
                    description:
                        - The Storage Account ManagementPolicies Rules.
                    type: list
                    returned: always
                    sample: [
                    {
                        "definition": {
                            "actions": {
                                "base_blob": {
                                    "delete": {
                                        "days_after_last_access_time_greater_than": 33.0,
                                        "days_after_modification_greater_than": 33.0
                                    },
                                    "enable_auto_tier_to_hot_from_cool": true,
                                    "tier_to_archive": {
                                        "days_after_last_access_time_greater_than": 33.0,
                                        "days_after_modification_greater_than": 33.0
                                    },
                                    "tier_to_cool": {
                                        "days_after_last_access_time_greater_than": 33.0,
                                        "days_after_modification_greater_than": 33.0
                                    }
                                },
                                "snapshot": {
                                    "delete": {
                                        "days_after_creation_greater_than": 33.0
                                    },
                                    "tier_to_archive": {
                                        "days_after_creation_greater_than": 33.0
                                    },
                                    "tier_to_cool": {
                                        "days_after_creation_greater_than": 33.0
                                    }
                                },
                                "version": {
                                    "delete": {
                                        "days_after_creation_greater_than": 33.0
                                    },
                                    "tier_to_archive": {
                                        "days_after_creation_greater_than": 33.0
                                    },
                                    "tier_to_cool": {
                                        "days_after_creation_greater_than": 33.0
                                    }
                                }
                            },
                            "filters": {
                                "blob_index_match": [
                                    {
                                        "name": "tags3",
                                        "op": "==",
                                        "value": "value3"
                                    }
                                ],
                                "blob_types": [
                                    "blockBlob",
                                    "appendBlob"
                                ],
                                "prefix_match": [
                                    "olcmtestcontainer2"
                                ]
                            }
                        },
                        "enabled": false,
                        "name": "olcmtest5",
                        "type": "Lifecycle"
                    }
                ]
'''


from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
try:
    from azure.core.exceptions import ResourceNotFoundError
except Exception:
    # This is handled in azure_rm_common
    pass


class AzureRMStorageAccountManagementPolicyInfo(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            resource_group=dict(required=True, type='str', aliases=['resource_group_name']),
            storage_account_name=dict(type='str', required=True),
        )

        self.results = dict(
            changed=False,
            state=dict()
        )

        self.resource_group = None
        self.storage_account_name = None
        self.state = None
        self.rules = None

        super(AzureRMStorageAccountManagementPolicyInfo, self).__init__(self.module_arg_spec,
                                                                        supports_tags=False,
                                                                        supports_check_mode=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        self.results['state'] = self.get_management_policy()

        return self.results

    def get_management_policy(self):
        self.log('Get info for storage account management policy')

        response = None
        try:
            response = self.storage_client.management_policies.get(self.resource_group, self.storage_account_name, 'default')
        except ResourceNotFoundError as ec:
            self.log("Failed to obtain the storage acount management policy, detail as {0}".format(ec))
            return

        return self.format_to_dict(response)

    def format_to_dict(self, obj):
        result = dict()
        result['id'] = obj.id
        result['resource_group'] = self.resource_group
        result['storage_account_name'] = self.storage_account_name
        result['name'] = obj.name
        result['type'] = obj.type
        result['last_modified_time'] = obj.last_modified_time
        result['policy'] = dict(rules=[])
        if obj.policy is not None:
            result['policy'] = obj.policy.as_dict()

        return result


def main():
    AzureRMStorageAccountManagementPolicyInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: discovery_device_v1_info
short_description: Information module for Discovery Device V1
description:
- Get all Discovery Device V1.
- >
   Returns the network devices discovered for the given Discovery ID. Discovery ID can be obtained using the "Get
   Discoveries by range" API.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
  id:
    description:
    - Id path parameter. Discovery ID.
    type: str
  taskId:
    description:
    - TaskId query parameter.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Discovery GetDiscoveredNetworkDevicesByDiscoveryIdV1
  description: Complete reference of the GetDiscoveredNetworkDevicesByDiscoveryIdV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!get-discovered-network-devices-by-discovery-id
notes:
  - SDK Method used are
    discovery.Discovery.get_discovered_network_devices_by_discovery_id_v1,

  - Paths used are
    get /dna/intent/api/v1/discovery/{id}/network-device,

"""

EXAMPLES = r"""
- name: Get all Discovery Device V1
  cisco.dnac.discovery_device_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
    taskId: string
    id: string
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": [
        {
          "anchorWlcForAp": "string",
          "authModelId": "string",
          "avgUpdateFrequency": 0,
          "bootDateTime": "string",
          "cliStatus": "string",
          "duplicateDeviceId": "string",
          "errorCode": "string",
          "errorDescription": "string",
          "family": "string",
          "hostname": "string",
          "httpStatus": "string",
          "id": "string",
          "imageName": "string",
          "ingressQueueConfig": "string",
          "interfaceCount": "string",
          "inventoryCollectionStatus": "string",
          "inventoryReachabilityStatus": "string",
          "lastUpdated": "string",
          "lineCardCount": "string",
          "lineCardId": "string",
          "location": "string",
          "locationName": "string",
          "macAddress": "string",
          "managementIpAddress": "string",
          "memorySize": "string",
          "netconfStatus": "string",
          "numUpdates": 0,
          "pingStatus": "string",
          "platformId": "string",
          "portRange": "string",
          "qosStatus": "string",
          "reachabilityFailureReason": "string",
          "reachabilityStatus": "string",
          "role": "string",
          "roleSource": "string",
          "serialNumber": "string",
          "snmpContact": "string",
          "snmpLocation": "string",
          "snmpStatus": "string",
          "softwareVersion": "string",
          "tag": "string",
          "tagCount": 0,
          "type": "string",
          "upTime": "string",
          "vendor": "string",
          "wlcApDeviceStatus": "string"
        }
      ],
      "version": "string"
    }
"""
